using System;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Text.RegularExpressions;

public class AuthorClass
{
    private string _original_Au_id;
    private string _au_id;
    private string _au_fname;
    private string _au_lname;
    private string _phone;
    private string _address;
    private string _city;
    private string _state;
    private string _zip;
    private bool _contract;

    private bool _dirtied = true;
    private string _connStr = null;
    private bool _enableValidation = true;

    public bool EnableValidation {
        get { return _enableValidation; }
        set { _enableValidation = value; }
    }
    public string ConnectionString {
        get { return _connStr; }
        set { _connStr = value; }
    }
    public string Au_id
    {
        get { return _au_id; }
        set {
            string newValue = value;
            if (_au_id != newValue) {
                if (_enableValidation) {
                    if (newValue == null)
                        throw new ArgumentException("Au_id null   ϴ.");

                    bool valOK = false;
                    newValue = newValue.Trim();
                    if (value.Trim().Length == 11) {
                        Regex rx = new Regex(@"\d{3}-\d{2}-\d{4}");
                        if (rx.IsMatch(newValue))
                            valOK = true;
                    }
                    else if (value.Trim().Length == 9) {
                        Regex rx = new Regex(@"\d{9}");
                        if (rx.IsMatch(newValue)) {
                            valOK = true;
                            newValue = newValue.Substring(0, 3) + "-"
                                + newValue.Substring(3, 2) + "-" 
                                + newValue.Substring(5);
                        }
                    }

                    if (!valOK)
                        throw new ArgumentException("Au_id  ߸Ǿϴ. 밡  '999-99-999' Ǵ '999999999'Դϴ.  Ŀ '9' 0 9 Դϴ.");
                }
                _au_id = newValue;
                _dirtied = true;
            }
        }
    }

    public string Original_au_id {
        get { return _original_Au_id; }
        set {
            string newValue = value;
            if (_original_Au_id != newValue)
                _original_Au_id = newValue;
        }
    }

    public string Au_fname
    {
        get { return _au_fname; }
        set {
            string newValue = value;
            if (_au_fname != newValue) {
                if (_enableValidation) {
                    if (newValue == null)
                        throw new ArgumentException("Au_fname null   ϴ.");

                    newValue = newValue.Trim();
                    if (newValue.Equals(string.Empty))
                        throw new ArgumentException("Au_fname ڿ Ҽ ϴ.");
                }
                _au_fname = newValue;
                _dirtied = true;
            }
        }
    }
    public String Au_lname
    {
        get { return _au_lname; }
        set { 
            string newValue = value;
            if (_au_lname != newValue) {
                if (_enableValidation) {
                    if (newValue == null)
                        throw new ArgumentException("Au_lname null   ϴ.");

                    newValue = newValue.Trim();
                    if (newValue.Equals(string.Empty))
                        throw new ArgumentException("Au_lname ڿ Ҽ ϴ.");
                }
                _au_lname = newValue;
                _dirtied = true;
            }
        }
    }
    public string Phone
    {
        get { return _phone; }
        set {
            string newValue = value;
            if (_phone != newValue) {
                if (_enableValidation) {
                    if (newValue == null)
                        throw new ArgumentException("Au_id null   ϴ.");

                    newValue = newValue.Trim();
                    Regex rx = new Regex(@"((\(\d{3}\) ?)|(\d{3}-))?\d{3}-\d{4}");
                    if (!rx.IsMatch(newValue))
                        throw new ArgumentException("Phone  ߸Ǿϴ. ̱ ȭȣ(: 415 548-7723) Էϼ.");
                }
                _phone = newValue;
                _dirtied = true;
            }

        }
    }
    public string Address
    {
        get { return _address; }
        set { _address = value; }
    }
    public string City
    {
        get { return _city; }
        set { _city = value; }
    }
    public string State
    {
        get { return _state; }
        set { _state = value; }
    }
    public string Zip
    {
        get { return _zip; }
        set { _zip = value; }
    }
    public bool Contract
    {
        get { return _contract; }
        set { _contract = value; }
    }
    public string FullName
    {
        get { return Au_fname + " " + Au_lname; }
    }

    public AuthorClass() {}

    public AuthorClass(bool enableValidation, string original_Au_id, string au_id, 
        string au_fname, string au_lname, string phone, string address, 
        string city, string state, string zip, bool contract)
    {
        EnableValidation = _enableValidation;
        Original_au_id = _original_Au_id;
        Au_id = au_id;
        Au_fname = au_fname;
        Au_lname = au_lname;
        Phone = phone;
        Address = address;
        City = city;
        State = state;
        Zip = zip;
        Contract = contract;
    }

    public AuthorClass(string au_id, string connStr)
    {
        _connStr = connStr;
        if (!Fill(au_id))
            throw new ArgumentException(au_id + " ġϴ  ã  ϴ.");
    }

    public bool Fill(string au_id) {
        if (!_dirtied)
            return true;

        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand sc =
            new SqlCommand(" SELECT * " +
                            " FROM authors " +
                            " WHERE au_id = @au_id",
                            conn);
        // Add the employee ID parameter and set its value.
        sc.Parameters.Add(new SqlParameter("@au_id", SqlDbType.Char, 11)).Value = au_id;
        SqlDataReader sdr = null;

        try {
            conn.Open();
            sdr = sc.ExecuteReader();

            // This is not a while loop. It only loops once.
            if (sdr != null && sdr.Read())  {
                Original_au_id = Au_id = sdr["au_id"].ToString();
                Au_fname = sdr["au_fname"].ToString();
                Au_lname = sdr["au_lname"].ToString();
                Phone = sdr["phone"].ToString();
                Address = sdr["address"].ToString();
                City = sdr["city"].ToString();
                State = sdr["state"].ToString();
                Zip = sdr["zip"].ToString();
                Contract = (bool)sdr["contract"];

                _dirtied = false;

                return true;
            }
            else {
                return false;
            }
        }
        finally {
            if (sdr != null) 
                sdr.Close();
            conn.Close();
        }
    }

    public int Insert() {
        return Save("Insert");
    }

    public void Update() {
        Save("Update");
    }

    /*
CREATE PROCEDURE SaveAuthor (
    @original_au_id char(11),
    @au_id char(11),
    @au_fname varchar(40),
    @au_lname varchar(20),
    @phone char(12),
    @address varchar(40),
    @city varchar(20),
    @state char(2),
    @zip char(5),
    @contract bit     
) AS

if (@original_au_id = '')
begin
	if not exists(
		select * 
			from [authors] 
			where [au_id]=@au_id
	)
	begin
		INSERT INTO [authors]
				   ([au_id]
				   ,[au_lname]
				   ,[au_fname]
				   ,[phone]
				   ,[address]
				   ,[city]
				   ,[state]
				   ,[zip]
				   ,[contract])
			 VALUES
				   (@au_id
				   ,@au_lname
				   ,@au_fname
				   ,@phone
				   ,@address
				   ,@city
				   ,@state
				   ,@zip
				   ,@contract)
		return @@IDENTITY
	end
	else
	begin
		return -1
	end
end
else
begin
	UPDATE [authors]
	   SET [au_id] = @au_id
		  ,[au_lname] = @au_lname
		  ,[au_fname] = @au_fname
		  ,[phone] = @phone
		  ,[address] = @address
		  ,[city] = @city
		  ,[state] = @state
		  ,[zip] = @zip
		  ,[contract] = @contract
	 WHERE [au_id]=@original_au_id
	return -1
end  
     */
    private int Save(string mode)
    {
        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand("SaveAuthor", conn);
        SqlTransaction tran; 
        cmd.CommandType = CommandType.StoredProcedure;

        if (mode.Equals("Insert"))
            cmd.Parameters.Add("@original_au_id", SqlDbType.Char, 11).Value = string.Empty;
        else
            cmd.Parameters.Add("@original_au_id", SqlDbType.Char, 11).Value = Original_au_id;
        cmd.Parameters.Add("@au_id", SqlDbType.Char, 11).Value = Au_id;
        cmd.Parameters.Add("@au_fname", SqlDbType.VarChar, 40).Value = Au_fname;
        cmd.Parameters.Add("@au_lname", SqlDbType.VarChar, 20).Value = Au_lname;
        cmd.Parameters.Add("@phone", SqlDbType.Char, 12).Value = Phone;
        cmd.Parameters.Add("@address", SqlDbType.VarChar, 40).Value = Address;
        cmd.Parameters.Add("@city", SqlDbType.VarChar, 20).Value = City;
        cmd.Parameters.Add("@state", SqlDbType.Char, 2).Value = State;
        cmd.Parameters.Add("@zip", SqlDbType.Char, 5).Value = Zip;
        cmd.Parameters.Add("@contract", SqlDbType.Bit).Value = Contract;
        SqlParameter parm = cmd.Parameters.Add("@RETURN_VALUE", SqlDbType.VarChar);
        parm.Direction = ParameterDirection.ReturnValue;

        conn.Open();
        tran = conn.BeginTransaction();
        cmd.Transaction = tran;

        int auctionState = -1;
        try {
            int effextedRow = cmd.ExecuteNonQuery();
            /*
            * Update Ǿ  auctionState -1 ̴.
            * Insert Ǿ  auctionState ߰  ĺ ̴.
            * */
            auctionState = (int)parm.Value;
            if (mode.Equals("Insert")) {
                if (auctionState == -1)
                    throw new Exception("au_id(" + Au_id + ")   ̹ մϴ.");
            }
            else {
                if (effextedRow != 1)
                    throw new Exception("au_id " + Au_id + "  ã ϴ.");
            }
            tran.Commit();
        }
        catch (Exception e) {
            tran.Rollback();
            throw;
        }
        finally {
            conn.Close();
        }

        return auctionState;
    }
}
