using System;
using System.Data;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.Configuration;

public class AuthorsComponent
{
    private string _connStr;
    private int _rowCount;

    public AuthorsComponent()
    {
        _connStr = ConfigurationManager.ConnectionStrings["pubsConnectionString"].ConnectionString;
    }

    public List<AuthorClass> GetAuthorsByState(String state, String sortExpression)
    {
        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand();
        SqlDataReader sdr;
        string queryString;

        if (state.Equals("*"))
            queryString = "SELECT * FROM [authors]";
        else {
            queryString = "SELECT * FROM [authors] WHERE (state = @state)";
            cmd.Parameters.Add(new SqlParameter("@state", SqlDbType.Char, 2)).Value = state;
        }
        cmd.Connection = conn;
        cmd.CommandText = queryString;

        conn.Open();
        sdr = cmd.ExecuteReader();
        List<AuthorClass> authors = new List<AuthorClass>();
        AuthorClass author;
        while (sdr.Read()) {
            author = new AuthorClass(false, "", (string)sdr["au_id"],
                (string)sdr["au_fname"], (string)sdr["au_lname"], 
                (string)sdr["phone"], (string)sdr["address"], 
                (string)sdr["city"], (string)sdr["state"], 
                (string)sdr["zip"], (bool)sdr["contract"]);
            authors.Add(author);
        }
        sdr.Close();
        conn.Close();

        if (sortExpression != null)
            authors.Sort(new AuthorComparer(sortExpression));
        return authors;
    }

    public List<AuthorClass> GetAuthorsByState(String state, 
        int maximumRows, int startRowIndex, String sortExpression) 
    {
        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand();
        SqlDataReader sdr;

        string queryString;
        string where;
        string orderBy1, orderBy2;
        int rowCount = GetRowCount(state);
        int lastRow = startRowIndex + maximumRows < rowCount 
                ? startRowIndex + maximumRows : rowCount;
        int resultRow = rowCount == lastRow 
                ? rowCount % maximumRows : maximumRows;

        _rowCount = rowCount;

        if (string.IsNullOrEmpty(sortExpression)) {
            orderBy1 = "order by au_id";
            orderBy2 = "order by au_id DESC";
            sortExpression = "au_id";
        }
        else {
            int descPos = sortExpression.ToLowerInvariant().LastIndexOf("desc");
            if (descPos > -1) {
                string columnName = sortExpression.Substring(0, descPos - 1);
                orderBy1 = "order by " + columnName + " DESC";
                orderBy2 = "order by " + columnName;
            }
            else {
                orderBy1 = "order by " + sortExpression;
                orderBy2 = "order by " + sortExpression + " DESC";
            }
        }

        if (state.Equals("*"))
            where = "";
        else
            where = " WHERE (state = '" + state + "')";
        queryString = "select top " + resultRow + " * from ";
        queryString += "    (select top " + lastRow + " * from [authors] " + where + " " + orderBy1 + ") a ";
        queryString += "    " + orderBy2;

        cmd.Connection = conn;
        cmd.CommandText = queryString;

        conn.Open();
        sdr = cmd.ExecuteReader();
        List<AuthorClass> authors = new List<AuthorClass>();
        AuthorClass author;
        while (sdr.Read()) {
            author = new AuthorClass(false, "", (string)sdr["au_id"],
                (string)sdr["au_fname"], (string)sdr["au_lname"],
                (string)sdr["phone"], (string)sdr["address"],
                (string)sdr["city"], (string)sdr["state"],
                (string)sdr["zip"], (bool)sdr["contract"]);
            authors.Add(author);
        }
        sdr.Close();
        conn.Close();

        authors.Sort(new AuthorComparer(sortExpression.ToLowerInvariant()));
        return authors;
    }

    public int GetRowCount(string state, string sortExpression) {
        return _rowCount;
    }

    private int GetRowCount(string state) {
        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand();
        SqlDataReader sdr;
        string queryString;

        if (state.Equals("*"))
            queryString = "SELECT count(*) FROM [authors]";
        else
        {
            queryString = "SELECT count(*) FROM [authors] WHERE (state = @state)";
            cmd.Parameters.Add(new SqlParameter("@state", SqlDbType.Char, 2)).Value = state;
        }
        cmd.Connection = conn;
        cmd.CommandText = queryString;

        conn.Open();
        int count = (int)cmd.ExecuteScalar();
        conn.Close();

        return count;
    }

    public List<String> GetStates()
    {
        List<String> states = new List<String>();

        string queryString = "SELECT DISTINCT state FROM [authors]";
        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand(queryString, conn);
        SqlDataReader sdr;

        conn.Open();
        sdr = cmd.ExecuteReader();
        while (sdr.Read())
            states.Add((string)sdr["state"]);

        sdr.Close();
        conn.Close();

        return states;
    }

    public AuthorClass GetAuthor(string au_id) {
        if (au_id == null)
            return null;
        else
            return new AuthorClass(au_id, _connStr);
    }

    public int UpdateAuthor(string original_Au_id, string au_lname, 
        string au_fname, string phone, string address,
        string city, string state, string zip, bool contract)
    {
        AuthorClass author = new AuthorClass();
        author.Original_au_id = author.Au_id = original_Au_id;
        author.Au_lname = au_lname;
        author.Au_fname = au_fname;
        author.Phone = phone;
        author.Address = address;
        author.City = city;
        author.State = state;
        author.Zip = zip;
        author.Contract = contract;
        author.ConnectionString = _connStr;
        author.Update();

        return 1;
    }

    public int UpdateAuthor(string original_au_id, string au_id, 
        string au_lname, string au_fname, string state)
    {
        AuthorClass author = new AuthorClass(original_au_id, _connStr);
        author.Au_id = au_id;
        author.Au_lname = au_lname;
        author.Au_fname = au_fname;
        author.State = state;
        author.Update();

        return 1;
    }

    public int UpdateAuthor(AuthorClass author)
    {
        author.Original_au_id = author.Au_id;
        author.ConnectionString = _connStr;
        author.Update();

        return 1;
    }

    public int InsertAuthor(string au_id, string au_lname,
        string au_fname, string phone, string address,
        string city, string state, string zip, bool contract)
    {
        AuthorClass author = new AuthorClass();
        author.Au_id = au_id;
        author.Au_lname = au_lname;
        author.Au_fname = au_fname;
        author.Phone = phone;
        author.Address = address;
        author.City = city;
        author.State = state;
        author.Zip = zip;
        author.Contract = contract;
        author.ConnectionString = _connStr;
        return author.Insert();
    }

    public int InsertAuthor(AuthorClass author)
    {
        author.ConnectionString = _connStr;
        return author.Insert();
    }

    public int DeleteAuthor(string original_Au_id)
    {
        string sql = "delete from [authors] where au_id = @au_id";
        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand(sql, conn);

        cmd.Parameters.Add("@au_id", SqlDbType.Char, 11).Value = original_Au_id;
        conn.Open();

        int deletedCount = -1;
        try {
            deletedCount = cmd.ExecuteNonQuery();
        }
        finally {
            conn.Close();
        }
        return deletedCount;
    }

    public int DeleteAuthor(AuthorClass author)
    {
        return DeleteAuthor(author.Au_id);
    }
}

public class AuthorComparer : IComparer<AuthorClass>
{
    private string _sortColumn;
    private bool _reverse;

    public AuthorComparer(string sortExpression)
    {
        _reverse = sortExpression.ToLowerInvariant().EndsWith(" desc");
        if (_reverse)
            _sortColumn = sortExpression.Substring(0, sortExpression.Length - 5);
        else
            _sortColumn = sortExpression;
    }

    public int Compare(AuthorClass a, AuthorClass b)
    {
        int retVal = 0;
        switch (_sortColumn)
        {
            case "au_id":
                retVal = String.Compare(a.Au_id, b.Au_id);
                break;
            case "au_fname":
                retVal = String.Compare(a.Au_fname, b.Au_fname);
                break;
            case "au_lname":
                retVal = String.Compare(a.Au_lname, b.Au_lname);
                break;
            case "state":
                retVal = String.Compare(a.State, b.State);
                break;
        }
        return (retVal * (_reverse ? -1 : 1));
    }
}
