using System;
using System.Data;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.Configuration;

public class BoardCompont
{
    private string _connStr;
    private int _rowCount;

    public BoardCompont()
    {
        _connStr = ConfigurationManager.ConnectionStrings["MySqlConnection"].ConnectionString;
    }

    public List<ItemClass> GetItems(BoardClass board, String sortExpression)
    {
        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand();
        SqlDataReader sdr;
        string queryString;

        queryString = "SELECT * FROM [" + board.TableName + "]";
        cmd.Connection = conn;
        cmd.CommandText = queryString;

        conn.Open();
        sdr = cmd.ExecuteReader();
        List<ItemClass> authors = new List<ItemClass>();
        ItemClass author;
        while (sdr.Read()) {
            author = new ItemClass(false, board.No, (int)sdr["no"],
                (string)sdr["writerName"], (string)sdr["title"], 
                (string)sdr["content"], (int)sdr["readCount"], 
                (DateTime)sdr["regDate"], (string)sdr["allowRoles"], 
                (string)sdr["allowUsers"]);
            authors.Add(author);
        }
        sdr.Close();
        conn.Close();

        if (sortExpression != null)
            authors.Sort(new ItemComparer(sortExpression));
        return authors;
    }

    public List<ItemClass> GetRecentItems(BoardClass board, int topCount) {
        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand();
        SqlDataReader sdr;
        string queryString;

        queryString = "SELECT top " + topCount + " * FROM [" + board.TableName + "] order by [RegDate] desc";
        cmd.Connection = conn;
        cmd.CommandText = queryString;

        conn.Open();
        sdr = cmd.ExecuteReader();
        List<ItemClass> authors = new List<ItemClass>();
        ItemClass author;
        while (sdr.Read())
        {
            author = new ItemClass(false, board.No, (int)sdr["no"],
                (string)sdr["writerName"], (string)sdr["title"],
                (string)sdr["content"], (int)sdr["readCount"],
                (DateTime)sdr["regDate"], (string)sdr["allowRoles"],
                (string)sdr["allowUsers"]);
            authors.Add(author);
        }
        sdr.Close();
        conn.Close();

        return authors;
    }
    

    private int GetRowCount(BoardClass board)
    {
        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand();
        string queryString;

        queryString = "SELECT count(*) FROM [" + board.TableName + "]";
        cmd.Connection = conn;
        cmd.CommandText = queryString;

        conn.Open();
        int count = (int)cmd.ExecuteScalar();
        conn.Close();

        return count;
    }

    public ItemClass GetItem(BoardClass board, int no)
    {
        if (no == 0)
            return null;
        else
            return new ItemClass(board, no, _connStr);
    }

    public int UpdateItem(BoardClass board, int no,
        string title, string content, 
        string allowRoles, string allowUsers)
    {
        ItemClass item = new ItemClass(board, no, _connStr);
        item.Title = title;
        item.Content = content;
        item.AllowRoles = allowRoles;
        item.AllowUsers = allowUsers;
        item.Update();

        return 1;
    }

    public int InsertItem(BoardClass board, 
        string writerName, string title, string content, 
        string allowRoles, string allowUsers)
    {
        ItemClass item = new ItemClass();
        item.Board = board;
        item.No = -1;
        item.WriterName = writerName;
        item.Title = title;
        item.Content = content;
        item.AllowUsers = allowUsers;
        item.AllowRoles = allowRoles;
        item.ConnectionString = _connStr;
        return item.Insert();
    }

    public int DeleteItem(BoardClass board, int no)
    {
        string sql = "delete from [" + board.TableName + "] where no = @no";
        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand(sql, conn);

        cmd.Parameters.Add("@no", SqlDbType.Int).Value = no;
        conn.Open();

        int deletedCount = -1;
        try {
            deletedCount = cmd.ExecuteNonQuery();
        }
        finally {
            conn.Close();
        }
        return deletedCount;
    }

    public void IncreaseReadCount(BoardClass board, int no) {
        ItemClass item = new ItemClass();
        item.Board = board;
        item.No = no;
        item.ConnectionString = _connStr;
        item.IncreaseReadCount();
    }
}

public class ItemComparer : IComparer<ItemClass>
{
    private string _sortColumn;
    private bool _reverse;

    public ItemComparer(string sortExpression)
    {
        _reverse = sortExpression.ToLowerInvariant().EndsWith(" desc");
        if (_reverse)
            _sortColumn = sortExpression.Substring(0, sortExpression.Length - 5);
        else
            _sortColumn = sortExpression;
    }

    public int Compare(ItemClass a, ItemClass b)
    {
        int retVal = 0;
        switch (_sortColumn) {
            case "no":
                retVal = String.Compare(a.No.ToString(), b.No.ToString());
                break;
            case "writerName":
                retVal = String.Compare(a.WriterName, b.WriterName);
                break;
            case "regDate":
                retVal = String.Compare(a.RegDate.ToLongTimeString(), b.RegDate.ToLongTimeString());
                break;
            case "title":
                retVal = String.Compare(a.Title, b.Title);
                break;
            case "readCount":
                retVal = String.Compare(a.ReadCount.ToString(), b.ReadCount.ToString());
                break;
        }
        return (retVal * (_reverse ? -1 : 1));
    }
}
