using System;
using System.IO;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;

namespace MyWebParts {
    public class ImageLinkEditorPart : EditorPart
    {
        DropDownList _ddlImages = null;
        DropDownList _ddlTargets = null;
        TextBox _txtTitle = null;
        TextBox _txtNavigateUrl = null;
        TextBox _txtImageUrl = null;
        CheckBox _chkDirectImage = null;
        Label _lblTitle = null;
        Label _lblNavigateUrl = null;
        Label _lblTarget = null;
        Label _lblImageUrl = null;
        Label _lblError = null;

        public ImageLinkEditorPart() {
            this.ID = "ImageLinkEditorPart";
            this.Title = "ImageLink Ʈ ";
        }

        protected override void CreateChildControls()
        {
            Controls.Clear();

            _ddlImages = new DropDownList();
            _ddlTargets = new DropDownList();
            _txtTitle = new TextBox();
            _txtNavigateUrl = new TextBox();
            _txtImageUrl = new TextBox();
            _chkDirectImage = new CheckBox();

            _lblTitle = new Label();
            _lblNavigateUrl = new Label();
            _lblTarget = new Label();
            _lblImageUrl = new Label();

            _lblError = new Label();
            _lblError.ForeColor = System.Drawing.Color.Red;
            _lblError.Visible = false;

            _chkDirectImage.Text = "Է";
            _chkDirectImage.AutoPostBack = true;
            _chkDirectImage.CheckedChanged += new EventHandler(chkDirectImage_CheckedChanged);

            this.Controls.Add(_ddlImages);
            this.Controls.Add(_ddlTargets);
            this.Controls.Add(_txtTitle);
            this.Controls.Add(_txtNavigateUrl);
            this.Controls.Add(_txtImageUrl);
            this.Controls.Add(_chkDirectImage);

            this.Controls.Add(_lblTitle);
            this.Controls.Add(_lblNavigateUrl);
            this.Controls.Add(_lblTarget);
            this.Controls.Add(_lblImageUrl);
        }

        protected override void Render(HtmlTextWriter writer)
        {
            AddAttributesToRender(writer);

            writer.AddAttribute(HtmlTextWriterAttribute.Border, "0", false);
            writer.RenderBeginTag(HtmlTextWriterTag.Table);

            writer.RenderBeginTag(HtmlTextWriterTag.Tr);
            writer.AddAttribute(HtmlTextWriterAttribute.Width, "100px");
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            _lblTitle.RenderControl(writer);
            writer.RenderEndTag();
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            _txtTitle.RenderControl(writer);
            writer.RenderEndTag();
            writer.RenderEndTag();

            writer.RenderBeginTag(HtmlTextWriterTag.Tr);
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            _lblNavigateUrl.RenderControl(writer);
            writer.RenderEndTag();
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            _txtNavigateUrl.RenderControl(writer);
            writer.RenderEndTag();
            writer.RenderEndTag();

            writer.RenderBeginTag(HtmlTextWriterTag.Tr);
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            _lblTarget.RenderControl(writer);
            writer.RenderEndTag();
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            _ddlTargets.RenderControl(writer);
            writer.RenderEndTag();
            writer.RenderEndTag();

            writer.RenderBeginTag(HtmlTextWriterTag.Tr);
            writer.AddAttribute(HtmlTextWriterAttribute.Valign, "top");
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            _lblImageUrl.RenderControl(writer);
            writer.RenderEndTag();
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            _ddlImages.RenderControl(writer);
            _txtImageUrl.RenderControl(writer);
            writer.Write("<br />");
            _chkDirectImage.RenderControl(writer);
            writer.RenderEndTag();
            writer.RenderEndTag();

            writer.RenderEndTag();
        }

        private void chkDirectImage_CheckedChanged(object sender, EventArgs e)
        {
            _txtImageUrl.Visible = _chkDirectImage.Checked;
            _ddlImages.Visible = !_chkDirectImage.Checked;
        }

        public override void SyncChanges()
        {
            EnsureChildControls();

            ImageLink2 imageLink = WebPartToEdit as ImageLink2;
            if (imageLink != null) {
                _ddlTargets.Items.Clear();
                foreach (string target in imageLink.Targets) {
                    _ddlTargets.Items.Add(target);
                }
                try {
                    _ddlTargets.SelectedValue = imageLink.Target.ToString();
                }
                catch {}
                _txtTitle.Text = imageLink.Title;
                _txtNavigateUrl.Text = imageLink.NavigateUrl;

                bool activateTextBox = imageLink.ImageInputType;
                if (!string.IsNullOrEmpty(imageLink.ImageRoot)) {
                    string basePath = this.Context.Server.MapPath(imageLink.ImageRoot);
                    try {
                        string[] files = Directory.GetFiles(basePath);
                        _ddlImages.Items.Clear();
                        foreach (string file in files) {
                            _ddlImages.Items.Add(imageLink.ImageRoot + file.Remove(0, basePath.Length));
                        }
                        try {
                            _ddlImages.SelectedValue = imageLink.ImageUrl;
                        }
                        catch { }
                    }
                    catch {
                        activateTextBox = true;
                        _chkDirectImage.Enabled = false;
                    }
                }
                else {
                    activateTextBox = true;
                    _chkDirectImage.Enabled = false;
                }

                _chkDirectImage.Checked = activateTextBox;
                _txtImageUrl.Text = imageLink.ImageUrl;
                if (activateTextBox) 
                    _ddlImages.Visible = false;
                else
                    _txtImageUrl.Visible = false;

                WebDisplayNameAttribute nameAtt;
                PropertyDescriptorCollection pdc;
                pdc = TypeDescriptor.GetProperties(imageLink);

                nameAtt = (WebDisplayNameAttribute)pdc["Title"].Attributes[typeof(WebDisplayNameAttribute)];
                _lblTitle.Text = nameAtt.DisplayName;
                nameAtt = (WebDisplayNameAttribute)pdc["ImageUrl"].Attributes[typeof(WebDisplayNameAttribute)];
                _lblImageUrl.Text = nameAtt.DisplayName;
                nameAtt = (WebDisplayNameAttribute)pdc["NavigateUrl"].Attributes[typeof(WebDisplayNameAttribute)];
                _lblNavigateUrl.Text = nameAtt.DisplayName;
                nameAtt = (WebDisplayNameAttribute)pdc["Target"].Attributes[typeof(WebDisplayNameAttribute)];
                _lblTarget.Text = nameAtt.DisplayName;
            }
        }

        public override bool ApplyChanges()
        {
            EnsureChildControls();
            ImageLink2 imageLink = WebPartToEdit as ImageLink2;
            if (imageLink != null) {
                try {
                    imageLink.Title = _txtTitle.Text;
                    imageLink.NavigateUrl = _txtNavigateUrl.Text;
                    imageLink.Target = _ddlTargets.SelectedValue;
                    if (_chkDirectImage.Checked) {
                        imageLink.ImageUrl = _txtImageUrl.Text;
                        imageLink.ImageInputType = true;
                    }
                    else {
                        imageLink.ImageUrl = _ddlImages.SelectedValue;
                        imageLink.ImageInputType = false;
                    }
                }
                catch {
                    _lblError.Text = " ߻ؼ ͸  ߽ϴ.";
                    _lblError.Visible = true;
                    return false;
                }
            }
            return true;
        }
    }
}