using System;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Collections.Generic;

public class BoardClass
{
    private int _no;
    private string _subject;
    private string _tableName;
    private string _allowRoles;
    private string _allowUsers;

    private string _connStr = null;

    public string ConnectionString
    {
        get { return _connStr; }
        set { _connStr = value; }
    }
    public int No {
        get { return _no; }
        set { _no = value; }
    }

    public string Subject
    {
        get { return _subject; }
        set {
            string newValue = value;
            if (_subject != newValue) {
                if (newValue == null)
                    throw new ArgumentException("Subject null   ϴ.");

                newValue = newValue.Trim();
                if (newValue.Equals(string.Empty))
                    throw new ArgumentException("Subject ڿ Ҽ ϴ.");
                _subject = newValue;
            }
        }
    }

    public string TableName
    {
        get { return _tableName; }
        set {
            string newValue = value;
            if (_tableName != newValue) {
                if (newValue == null)
                    throw new ArgumentException("TableName null   ϴ.");

                newValue = newValue.Trim();
                if (newValue.Equals(string.Empty))
                    throw new ArgumentException("TableName ڿ Ҽ ϴ.");
                _tableName = newValue;
            }
        }
    }

    public string AllowRoles {
        get {
            if (_allowRoles == null)
                _allowRoles = string.Empty;
            return _allowRoles; 
        }
        set { _allowRoles = value; }
    }
    public string AllowUsers {
        get {
            if (_allowUsers == null)
                _allowUsers = string.Empty;
            return _allowUsers; 
        }
        set { _allowUsers = value; }
    }

    public bool IsAuthenticatedUser {
        get {
            if (IsInRoles() || IsInUsers())
                return true;
            
            string adminRole = ConfigurationManager.AppSettings["BoardAdminRole"];
            if (!string.IsNullOrEmpty(adminRole))
                return Roles.IsUserInRole(adminRole);
            else
                return false;
        }
    }

	public BoardClass()	{}

    public BoardClass(int no, 
        string subject, string tableName, 
        string allowRoles, string allowUsers)
    {
        No = no;
        Subject = subject;
        TableName = tableName;
        AllowRoles = allowRoles;
        AllowUsers = allowUsers;
    }

    public BoardClass(int no, string connStr)
    {
        ConnectionString = connStr;
        if (!Fill(no))
            throw new ArgumentException(no + " ġϴ  ã  ϴ.");
    }

    public bool IsInRoles()
    {
        if (string.IsNullOrEmpty(AllowRoles))
            return false;

        HttpContext context = HttpContext.Current;

        foreach (String role in AllowRoles.Split(new char[] { ';' })) {
            if (!string.IsNullOrEmpty(role) && 
                (context.User.IsInRole(role)))
                return true;
        }

        return false;
    }

    public bool IsInRoles(string pRole)
    {
        if (string.IsNullOrEmpty(AllowRoles) 
            || string.IsNullOrEmpty(pRole))
            return false;

        foreach (String role in AllowRoles.Split(new char[] { ';' })) {
            if (!string.IsNullOrEmpty(role) &&
                pRole.Equals(role))
                return true;
        }

        return false;
    }

    public bool IsInUsers()
    {
        return IsInUsers(HttpContext.Current.User.Identity.Name, true, true);
    }

    public bool IsInUsers(string userName, bool checkAll, bool checkAuthUser)
    {
        if (string.IsNullOrEmpty(AllowUsers))
            return false;

        bool isAuthenticated = HttpContext.Current.Request.IsAuthenticated;
        foreach (String user in AllowUsers.Split(new char[] { ';' })) {
            if (!string.IsNullOrEmpty(user) &&
                ((checkAll && user.Equals("*"))
                || (checkAuthUser && user.Equals("?") && isAuthenticated)
                || user.Equals(userName)))
                return true;
        }

        return false;
    }

    public List<string> GetAvailableUsers()
    {
        List<string> users = new List<string>();

        if (!string.IsNullOrEmpty(AllowUsers)) {
            foreach (String user in AllowUsers.Split(new char[] { ';' })) {
                if (!string.IsNullOrEmpty(user)) {
                    if (!user.Equals("*") && !user.Equals("?"))
                        users.Add(user);
                    else {
                        foreach (MembershipUser oUser in Membership.GetAllUsers())
                            users.Add(oUser.UserName);

                        return users;
                    }
                }
            }
        }

        if (!string.IsNullOrEmpty(AllowRoles)) {
            foreach (String role in AllowRoles.Split(new char[] { ';' })) {
                foreach (String user in Roles.GetUsersInRole(role)) {
                    if (!users.Contains(user))
                        users.Add(user);
                }
            }
        }

        return users;
    }

    public bool Fill(int no) 
    {
        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand sc =
            new SqlCommand(" SELECT * " +
                            " FROM boardInfo " +
                            " WHERE no = @no",
                            conn);

        sc.Parameters.Add(new SqlParameter("@no", SqlDbType.Int)).Value = no;
        SqlDataReader sdr = null;

        try {
            conn.Open();
            sdr = sc.ExecuteReader();

            if (sdr != null && sdr.Read())  {
                No = (int)sdr["no"];
                _subject = (string)sdr["subject"];
                _tableName = (string)sdr["tableName"];
                _allowRoles = sdr["allowRoles"] == System.DBNull.Value ? null : (string)sdr["allowRoles"];
                _allowUsers = sdr["allowUsers"] == System.DBNull.Value ? null : (string)sdr["allowUsers"];

                return true;
            }
            else {
                return false;
            }
        }
        finally {
            if (sdr != null) 
                sdr.Close();
            conn.Close();
        }
    }

    public int Insert() 
    {
        string sqlInsertCmd = "INSERT INTO [boardInfo]"
            + " ([subject] ,[tableName], "
            + "[allowRoles] ,[allowUsers]) "
            + "VALUES (@subject, @tableName, "
            + "@allowRoles, @allowUsers)";

        string sqlCreateCmd = @"
CREATE TABLE [" + TableName + @"](
	[no] [int] IDENTITY(1,1) NOT NULL,
	[writerName] [varchar](50) COLLATE Korean_Wansung_CI_AS NOT NULL,
	[title] [varchar](200) COLLATE Korean_Wansung_CI_AS NOT NULL,
	[content] [text] COLLATE Korean_Wansung_CI_AS NOT NULL,
	[regDate] [datetime] NOT NULL CONSTRAINT [DF_" + TableName + @"_regDate]  DEFAULT (getdate()),
	[readCount] [int] NOT NULL CONSTRAINT [DF_" + TableName + @"_readCount]  DEFAULT ((0)),
	[allowRoles] [varchar](300) COLLATE Korean_Wansung_CI_AS NULL,
	[allowUsers] [varchar](300) COLLATE Korean_Wansung_CI_AS NULL,
    CONSTRAINT [PK_" + TableName + @"] PRIMARY KEY CLUSTERED 
    (
	    [no] ASC
    )WITH (IGNORE_DUP_KEY = OFF) ON [PRIMARY]
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
";

        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand();
        SqlTransaction tran;

        cmd.CommandType = CommandType.Text;
        conn.Open();
        tran = conn.BeginTransaction();
        cmd.Connection = conn;
        cmd.Transaction = tran;

        int auctionState = -1;
        try {
            cmd.CommandText = sqlInsertCmd;
            cmd.Parameters.Add("@subject", SqlDbType.VarChar, 50).Value = Subject;
            cmd.Parameters.Add("@tableName", SqlDbType.VarChar, 50).Value = TableName;
            cmd.Parameters.Add("@allowRoles", SqlDbType.VarChar, 300).Value = AllowRoles;
            cmd.Parameters.Add("@allowUsers", SqlDbType.VarChar, 300).Value = AllowUsers;
            cmd.ExecuteNonQuery();
            cmd.CommandText = sqlCreateCmd;
            cmd.Parameters.Clear();
            cmd.ExecuteNonQuery();

            tran.Commit();
            auctionState = 1;
        }
        catch (Exception err) {
            tran.Rollback();
            auctionState = -1;
        }
        finally {
            conn.Close();
        }

        return auctionState;
    }

    public int Update() 
    {
        string sqlCmd = "UPDATE [boardInfo]"
    + "SET [subject] = @subject, "
    + " [tableName] = @tableName,"
    + " [allowRoles] = @allowRoles,"
    + " [allowUsers] = @allowUsers "
    + "WHERE no=@no";

        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand(sqlCmd, conn);

        cmd.Parameters.Add("@no", SqlDbType.Int).Value = No;
        cmd.Parameters.Add("@subject", SqlDbType.VarChar, 50).Value = Subject;
        cmd.Parameters.Add("@tableName", SqlDbType.VarChar, 50).Value = TableName;
        cmd.Parameters.Add("@allowRoles", SqlDbType.VarChar, 300).Value = AllowRoles;
        cmd.Parameters.Add("@allowUsers", SqlDbType.VarChar, 300).Value = AllowUsers;

        cmd.CommandType = CommandType.Text;
        conn.Open();

        int auctionState;
        try {
            cmd.ExecuteNonQuery();
            auctionState = 1;
        }
        catch (Exception err) {
            auctionState = -1;
        }
        finally {
            conn.Close();
        }

        return auctionState;
    }
}
