using System;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Web;
using System.Web.Security;

public class ItemClass
{
    private int _boardNo = 0;
    private int _no;
    private string _writerName;
    private string _title;
    private string _content;
    private DateTime _regDate;
    private int _readCount;
    private string _allowRoles;
    private string _allowUsers;

    private bool _dirtied = true;
    private string _connStr = null;
    private bool _enableValidation = true;
    private BoardClass _board = null;

    public BoardClass Board {
        get {
            if (_board == null) {
                if (_boardNo == 0)
                    throw new InvalidOperationException("BoardNo  ʾҽϴ.");
                _board = new BoardClass(_boardNo, _connStr);
            }

            return _board; 
        }
        set { _board = value; }
    }

    public bool EnableValidation {
        get { return _enableValidation; }
        set { _enableValidation = value; }
    }
    public string ConnectionString {
        get { return _connStr; }
        set { _connStr = value; }
    }

    public int BoardNo {
        get { return _boardNo; }
        set { _boardNo = value; }
    }

    public int No {
        get { return _no; }
        set { _no = value; }
    }

    public string WriterName
    {
        get { return _writerName; }
        set {
            string newValue = value;
            if (_writerName != newValue) {
                if (_enableValidation) {
                    if (newValue == null)
                        throw new ArgumentException("WriterName null   ϴ.");

                    newValue = newValue.Trim();
                    if (newValue.Equals(string.Empty))
                        throw new ArgumentException("WriterName ڿ Ҽ ϴ.");
                }
                _writerName = newValue;
                _dirtied = true;
            }
        }
    }

    public string Title
    {
        get { return _title; }
        set {
            string newValue = value;
            if (_title != newValue) {
                if (_enableValidation) {
                    if (newValue == null)
                        throw new ArgumentException("Title null   ϴ.");

                    newValue = newValue.Trim();
                    if (newValue.Equals(string.Empty))
                        throw new ArgumentException("Title ڿ Ҽ ϴ.");
                }
                _title = newValue;
                _dirtied = true;
            }
        }
    }
    
    public String Content
    {
        get { return _content; }
        set { 
            string newValue = value;
            if (_content != newValue) {
                if (_enableValidation) {
                    if (newValue == null)
                        throw new ArgumentException("Content null   ϴ.");

                    newValue = newValue.Trim();
                    if (newValue.Equals(string.Empty))
                        throw new ArgumentException("Content ڿ Ҽ ϴ.");
                }
                _content = newValue;
                _dirtied = true;
            }
        }
    }
    public int ReadCount {
        get { return _readCount; }
        set {
            _readCount = value;
            _dirtied = true;
        }
    }
    public DateTime RegDate {
        get { return _regDate; }
        set {
            _regDate = value;
        }
    }
    public string AllowRoles {
        get {
            if (_allowRoles == null)
                _allowRoles = string.Empty;
            return _allowRoles; 
        }
        set { _allowRoles = value; }
    }
    public string AllowUsers {
        get {
            if (_allowUsers == null)
                _allowUsers = string.Empty;
            return _allowUsers; 
        }
        set { _allowUsers = value; }
    }

    public bool IsAuthenticatedUser {
        get { return IsInRoles() || IsInUsers() || IsModifier; }
    }

    public bool IsModifier
    {
        get {
            if (WriterName.Equals(HttpContext.Current.User.Identity.Name))
                return true;

            string adminRole = ConfigurationManager.AppSettings["BoardAdminRole"];
            if (!string.IsNullOrEmpty(adminRole))
                return Roles.IsUserInRole(adminRole);
            else
                return false;
        }
    }

    public ItemClass() {}

    public ItemClass(bool enableValidation, int boardNo, int no, 
        string writerName, string title,  string content, int readCount,
        DateTime regDate, string allowRoles, string allowUsers)
    {
        EnableValidation = _enableValidation;
        BoardNo = boardNo;
        No = no;
        WriterName = writerName;
        Title = title;
        Content = content;
        RegDate = regDate;
        ReadCount = readCount;
        AllowRoles = allowRoles;
        AllowUsers = allowUsers;
    }

    public ItemClass(int boardNo, int no, string connStr)
    {
        ConnectionString = connStr;
        BoardNo = boardNo;
        if (!Fill(no))
            throw new ArgumentException(no + " ġϴ  ã  ϴ.");
    }

    public ItemClass(BoardClass board, int no, string connStr)
    {
        ConnectionString = connStr;
        _board = board;
        if (!Fill(no))
            throw new ArgumentException(no + " ġϴ  ã  ϴ.");
    }

    public bool IsInRoles()
    {
        if (string.IsNullOrEmpty(AllowRoles))
            return false;

        HttpContext context = HttpContext.Current;

        foreach (String role in AllowRoles.Split(new char[] { ';' })) {
            if (!string.IsNullOrEmpty(role) && 
                (context.User.IsInRole(role)))
                return true;
        }

        return false;
    }

    public bool IsInRoles(string pRole)
    {
        if (string.IsNullOrEmpty(AllowRoles)
            || string.IsNullOrEmpty(pRole))
            return false;

        foreach (String role in AllowRoles.Split(new char[] { ';' })) {
            if (!string.IsNullOrEmpty(role) &&
                pRole.Equals(role))
                return true;
        }

        return false;
    }

    public bool IsInUsers()
    {
        string userName = HttpContext.Current.User.Identity.Name;
        return IsInUsers(userName, true, true);
    }

    public bool IsInUsers(string userName, bool checkAll, bool checkAuthUser)
    {
        if (string.IsNullOrEmpty(AllowUsers))
            return false;

        bool isAuthenticated = HttpContext.Current.Request.IsAuthenticated;
        foreach (String user in AllowUsers.Split(new char[] { ';' })) {
            if (!string.IsNullOrEmpty(user) &&
                ((checkAll && user.Equals("*"))
                || (checkAuthUser && user.Equals("?") && isAuthenticated)
                || user.Equals(userName)))
                return true;
        }

        return false;
    }

    public bool Fill(int no) 
    {
        if (!_dirtied)
            return true;

        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand sc =
            new SqlCommand(" SELECT * " +
                            " FROM " + Board.TableName + " " +
                            " WHERE no = @no",
                            conn);

        sc.Parameters.Add(new SqlParameter("@no", SqlDbType.Int)).Value = no;
        SqlDataReader sdr = null;

        try {
            conn.Open();
            sdr = sc.ExecuteReader();

            if (sdr != null && sdr.Read())  {
                No = (int)sdr["no"];
                WriterName = (string)sdr["writerName"];
                Title = (string)sdr["title"];
                Content = (string)sdr["content"];
                RegDate = (DateTime)sdr["regDate"];
                ReadCount = (int)sdr["readCount"];
                AllowRoles = (sdr["allowRoles"] == System.DBNull.Value) ? null : (string)sdr["allowRoles"];
                AllowUsers = (sdr["allowUsers"] == System.DBNull.Value) ? null : (string)sdr["allowUsers"];

                _dirtied = false;

                return true;
            }
            else {
                return false;
            }
        }
        finally {
            if (sdr != null) 
                sdr.Close();
            conn.Close();
        }
    }

    public int Insert() {

        string sqlCmd = "INSERT INTO [" + Board.TableName + "]"
            + " ([writerName] ,[title] ,[content] " 
            + ",[allowRoles] ,[allowUsers]) " 
            + "VALUES (@writerName, @title, @content, " 
            + "@allowRoles, @allowUsers)";

        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand(sqlCmd, conn);

        cmd.Parameters.Add("@writerName", SqlDbType.VarChar, 50).Value = WriterName;
        cmd.Parameters.Add("@title", SqlDbType.VarChar, 200).Value = Title;
        cmd.Parameters.Add("@content", SqlDbType.Text).Value = Content;
        cmd.Parameters.Add("@allowRoles", SqlDbType.VarChar, 300).Value = AllowRoles;
        cmd.Parameters.Add("@allowUsers", SqlDbType.VarChar, 300).Value = AllowUsers;

        cmd.CommandType = CommandType.Text;
        conn.Open();

        int auctionState = -1;
        try {
            cmd.ExecuteNonQuery();
            auctionState = 1;
        }
        catch (Exception err) {
            auctionState = -1;
        }
        finally {
            conn.Close();
        }

        return auctionState;
    }

    public int Update() 
    {
        string sqlCmd = "UPDATE [" + Board.TableName + "]"
            + "SET [writerName] = @writerName, "
            + " [title] = @title,"
            + " [content] = @content,"
            + " [allowRoles] = @allowRoles,"
            + " [allowUsers] = @allowUsers "
            + "WHERE no=@no";

        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand(sqlCmd, conn);

        cmd.Parameters.Add("@no", SqlDbType.Int).Value = No;
        cmd.Parameters.Add("@writerName", SqlDbType.VarChar, 50).Value = WriterName;
        cmd.Parameters.Add("@title", SqlDbType.VarChar, 200).Value = Title;
        cmd.Parameters.Add("@content", SqlDbType.Text).Value = Content;
        cmd.Parameters.Add("@allowRoles", SqlDbType.VarChar, 300).Value = AllowRoles;
        cmd.Parameters.Add("@allowUsers", SqlDbType.VarChar, 300).Value = AllowUsers;

        cmd.CommandType = CommandType.Text;
        conn.Open();

        int auctionState;
        try {
            cmd.ExecuteNonQuery();
            auctionState = 1;
        }
        catch (Exception err) {
            auctionState = -1;
        }
        finally {
            conn.Close();
        }

        return auctionState;
    }

    public void IncreaseReadCount() {
        string sqlCmd = "UPDATE [" + Board.TableName + "]"
    + "SET [readCount] = readCount + 1 "
    + "WHERE no=@no";

        SqlConnection conn = new SqlConnection(_connStr);
        SqlCommand cmd = new SqlCommand(sqlCmd, conn);

        cmd.Parameters.Add("@no", SqlDbType.Int).Value = No;

        cmd.CommandType = CommandType.Text;
        conn.Open();
        cmd.ExecuteNonQuery();
        conn.Close();
    }
}
