using System;
using System.Configuration;
using System.Drawing;
using System.Collections.Generic;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;

public class PieceBoardListPart : WebPart
{
    int _topCount = 3;
    int _boardNo = -1;
    BoardClass _board = null;
    Label _lblAccessDenide = null;
    GridView _gvList = null;

    private BoardClass Board {
        get {
            if (_board == null && _boardNo != -1) {
                string connStr = ConfigurationManager.ConnectionStrings["MySqlConnection"].ConnectionString;
                _board = new BoardClass(_boardNo, connStr);
            }
            return _board;
        }
    }

    [Personalizable]
    public int TopCount {
        get { return _topCount; }
        set { _topCount = value; }
    }

    [Personalizable]
    public int BoardNo {
        get { return _boardNo; }
        set { _boardNo = value; }
    }

    public override String Title {
        get {
            if (Board != null)
                return Board.Subject;
            else
                return "";
        }
        set { ; }
    }

    public PieceBoardListPart() { }

    protected override void CreateChildControls()
    {
        Controls.Clear();

        _lblAccessDenide = new Label();
        _lblAccessDenide.ID = "lblAccessDenide";
        _lblAccessDenide.Visible = false;
        _lblAccessDenide.ForeColor = Color.Red;
        _lblAccessDenide.Text = "ٱ ϴ";
        this.Controls.Add(_lblAccessDenide);

        _gvList = new GridView();
        _gvList.ID = "gvList";
        _gvList.AutoGenerateColumns = false;
        _gvList.RowDataBound += new GridViewRowEventHandler(gvList_RowDataBound);

        BoundField bf;
        bf = new BoundField();
        bf.DataField = "No";
        bf.HeaderText = "No";
        bf.SortExpression = "No";
        _gvList.Columns.Add(bf);

        bf = new BoundField();
        bf.DataField = "WriterName";
        bf.HeaderText = "ۼ";
        bf.SortExpression = "WriterName";
        _gvList.Columns.Add(bf);

        HyperLinkField hlf;
        hlf = new HyperLinkField();

        hlf.DataNavigateUrlFields = new string[] { "BoardNo", "No" };
        hlf.DataNavigateUrlFormatString = "~/Board/Content.aspx?board={0}&no={1}";
        hlf.DataTextField = "Title";
        hlf.HeaderText = "";
        hlf.SortExpression = "Title";
        hlf.ControlStyle.Width = new Unit(100);
        _gvList.Columns.Add(hlf);

        bf = new BoundField();
        bf.DataField = "ReadCount";
        bf.HeaderText = "ȸ";
        bf.SortExpression = "ReadCount";
        _gvList.Columns.Add(bf);

        bf = new BoundField();
        bf.DataField = "RegDate";
        bf.HeaderText = "¥";
        bf.SortExpression = "RegDate";
        bf.DataFormatString = "{0:yyyy-MM-dd}";
        _gvList.Columns.Add(bf);

        bf = new BoundField();
        bf.DataField = "AllowRoles";
        bf.HeaderText = "AllowRoles";
        bf.SortExpression = "AllowRoles";
        _gvList.Columns.Add(bf);

        bf = new BoundField();
        bf.DataField = "AllowUsers";
        bf.HeaderText = "AllowUsers";
        bf.SortExpression = "AllowUsers";
        _gvList.Columns.Add(bf);

        this.Controls.Add(_gvList);
    }

    private void gvList_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow) {
            ItemClass item = (ItemClass)e.Row.DataItem;
            if (!item.IsAuthenticatedUser)
                e.Row.Cells[2].Enabled = false;
        }
    }

    protected override void OnPreRender(EventArgs e)
    {
        BoardCompont bc = new BoardCompont();
        _gvList.DataSource = bc.GetRecentItems(Board, TopCount);
        _gvList.DataBind();
        base.OnPreRender(e);
    }

    public override EditorPartCollection CreateEditorParts()
    {
        List<EditorPart> customEditorPartCollection = new List<EditorPart>();
        customEditorPartCollection.Add(new PieceBoardListEditPart("PieceBoardListEditPart"));
        EditorPartCollection editorPartCollection = new EditorPartCollection(customEditorPartCollection);
        return editorPartCollection;
    }
}
