﻿<%@ Control Language="C#" ClassName="EditSiteMap" %>
<%@ Import Namespace="System.Configuration" %>
<%@ Import Namespace="SqlSiteMap" %>
<%@ Import Namespace="System.Data" %>
<%@ Import Namespace="System.Data.SqlClient" %>
<%@ Import Namespace="System.Collections.Generic" %>
<script runat="server">
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack) {
            CreateSiteMapTreeView();
        }
    }

    private string _selectedValue = null;
    private void CreateSiteMapTreeView() {
        SiteMapProviderBase prov = GetTempSqlSiteMapProvider();
        SiteMapNodeBase rootNode = (SiteMapNodeBase)prov.RootNode;

        _selectedValue = TreeView1.SelectedValue;
        TreeView1.Nodes.Clear();
        TreeNode treeRootNode = CreateTreeNode(rootNode);
        TreeView1.Nodes.Add(treeRootNode);
        CreateTreeNodes(rootNode, treeRootNode);
        TreeView1.ExpandAll();
    }

    private SiteMapProviderBase GetTempSqlSiteMapProvider()
    {
        NameValueCollection atts = new NameValueCollection();
        atts.Add("connectionStringName", ProviderHelper.GetConnectionStringName(SiteMap.Provider));
        atts.Add("boardRelativeDirectory", "~/board/");
        GeneralSiteMapProvider prov = new GeneralSiteMapProvider(false);
        prov.Initialize(SiteMap.Provider.Name, atts);

        return prov;
    }

    private TreeNode CreateTreeNode(SiteMapNodeBase node)
    {
        TreeNode treeNode = new TreeNode(node.Title, node.Key);
        treeNode.ImageUrl = node.IconFile;
        treeNode.ToolTip = node.Description;
        if (!node.Visible)
            treeNode.Text += "<font color='red'><i>(숨김)</i></font>";

        treeNode.Selected = !string.IsNullOrEmpty(_selectedValue)
            && _selectedValue.Equals(node.Key);
        return treeNode;
    }

    private void CreateTreeNodes(SiteMapNodeBase parentSsmn, TreeNode parentNode)
    {
        if (parentSsmn.HasChildNodes) {
            foreach (SiteMapNodeBase n in parentSsmn.ChildNodes) {
                TreeNode node = CreateTreeNode(n);

                parentNode.ChildNodes.Add(node);
                if (n.HasChildNodes)
                    CreateTreeNodes(n, node);
            }
        }
    }

    protected void TreeView1_SelectedNodeChanged(object sender, EventArgs e)
    {
        SiteMapProviderBase prov = GetTempSqlSiteMapProvider();
        SiteMapNodeBase selectedNode =
            (SiteMapNodeBase)prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);

        bool isRoot = false;
        if (prov.RootNode.Key.Equals(TreeView1.SelectedValue)) {
            isRoot = true;
        }
        else if (selectedNode == null) {
            pnlEditNode.Visible = false;
            return;
        }

        if (!isRoot && selectedNode.PreviousSibling != null) {
            ibtnUp.Enabled = true;
            ibtnUp.ImageUrl = "~/Images/blue13_up.gif";
        }
        else {
            ibtnUp.Enabled = false;
            ibtnUp.ImageUrl = "~/Images/gray05_up.gif";
        }

        if (!isRoot && selectedNode.PreviousSibling != null
            && ((INodeEditable)selectedNode.PreviousSibling).Parentable) {
            ibtnUpRight.Enabled = true;
            ibtnUpRight.ImageUrl = "~/Images/blue13_upRight.gif";
        }
        else {
            ibtnUpRight.Enabled = false;
            ibtnUpRight.ImageUrl = "~/Images/gray05_upRight.gif";            
        }

        if (!isRoot && selectedNode.NextSibling != null 
            && ((INodeEditable)selectedNode.NextSibling).Parentable) {
            ibtnDownRight.Enabled = true;
            ibtnDownRight.ImageUrl = "~/Images/blue13_downRight.gif";
        }
        else {
            ibtnDownRight.Enabled = false;
            ibtnDownRight.ImageUrl = "~/Images/gray05_downRight.gif";
        }

        if (!isRoot && selectedNode.NextSibling != null) {
            ibtnDown.Enabled = true;
            ibtnDown.ImageUrl = "~/Images/blue13_down.gif";
        }
        else {
            ibtnDown.Enabled = false;
            ibtnDown.ImageUrl = "~/Images/gray05_down.gif";
        }
        if (selectedNode.ParentNode != null 
            && selectedNode.ParentNode.ParentNode != null) {
            ibtnLeftUp.Enabled = true;
            ibtnLeftUp.ImageUrl = "~/Images/blue13_leftUp.gif";
            ibtnLeftDown.Enabled = true;
            ibtnLeftDown.ImageUrl = "~/Images/blue13_leftDown.gif";
        }
        else {
            ibtnLeftUp.Enabled = false;
            ibtnLeftUp.ImageUrl = "~/Images/gray05_leftUp.gif";
            ibtnLeftDown.Enabled = false;
            ibtnLeftDown.ImageUrl = "~/Images/gray05_leftDown.gif";
        }

        if (selectedNode is BoardSiteMapNode 
            && !string.IsNullOrEmpty(selectedNode.InstanceParam)) {
            string childNodesDisplayMode = null;
            string boardNo = null;
            foreach (string p in selectedNode.InstanceParam.Split(new char[] { ';' })) {
                if (!string.IsNullOrEmpty(p)) {
                    if (p.StartsWith("childNodesDisplayMode=")) {
                        childNodesDisplayMode = p.Remove(0, "childNodesDisplayMode=".Length);
                    }
                    else if (p.StartsWith("boardNo=")) {
                        boardNo = p.Remove(0, "boardNo=".Length);
                    }
                }
            }
            if (childNodesDisplayMode != null)
                ddlChildNodesDisplayMode.SelectedValue = childNodesDisplayMode;
            else
                ddlChildNodesDisplayMode.SelectedIndex = 0;

            ddlBoard.SelectedValue = boardNo;
        }
        else {
            txtTitle.Text = selectedNode.Title;
            txtUrl.Text = selectedNode.Url;
            txtDescription.Text = selectedNode.Description;
        }
        chkVisible.Checked = selectedNode.Visible;
        txtIconFile.Text = selectedNode.IconFile;
        
        string roles = string.Empty;
        if (selectedNode.Roles != null) {
            foreach (string role in selectedNode.Roles)
                if (string.IsNullOrEmpty(role)) {
                    if (roles.Equals(string.Empty))
                        roles = role;
                    else
                        roles += "," + role;
                }
        }
        txtRoles.Text = roles;
        hiddenID.Value = selectedNode.Id.ToString();
        pnlEditNode.Visible = true;
        rblNodeType.DataBind();
        if (selectedNode is BoardSiteMapNode)
            rblNodeType.SelectedIndex = 1;
        else
            rblNodeType.SelectedIndex = 0;
        rblNodeType.Enabled = !selectedNode.HasChildNodes;
        rblNodeType_SelectedIndexChanged(null, null);
    }

    protected void ibtnMove_Command(object sender, CommandEventArgs e)
    {
        switch (e.CommandName) {
            case "leftUp":
                MoveToUpLevel(true);
                break;
            case "leftDown":
                MoveToUpLevel(false);
                break;
            case "up":
                MoveUpDown(true);
                break;
            case "down":
                MoveUpDown(false);
                break;
            case "upRight":
                MoveToDownLevel(true);
                break;
            case "downRight":
                MoveToDownLevel(false);
                break;
        }
        CreateSiteMapTreeView();
        ProviderHelper.UpdateSiteMap(SiteMap.Provider);
        TreeView1_SelectedNodeChanged(null, null);
    }

    private void MoveToUpLevel(bool isUp)
    {
        SiteMapProviderBase prov = GetTempSqlSiteMapProvider();
        SiteMapNodeBase selectedNode = (SiteMapNodeBase)
            prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);

        if (selectedNode.ParentNode == null 
            || selectedNode.ParentNode.ParentNode == null)
            return;

        List<SiteMapNodeBase> nodes = new List<SiteMapNodeBase>();
        short idx = 1;
        SiteMapNodeBase parentNode = (SiteMapNodeBase)selectedNode.ParentNode;
        foreach (SiteMapNodeBase node in parentNode.ParentNode.ChildNodes) {
            node.MenuOrder = idx;
            nodes.Add(node);
            idx += 2;
        }
        if (isUp)
            selectedNode.MenuOrder = (short)(parentNode.MenuOrder - 1);
        else
            selectedNode.MenuOrder = (short)(parentNode.MenuOrder + 1);

        selectedNode.ParentId = parentNode.ParentId;
        nodes.Add(selectedNode);
        nodes.Sort(new SiteMapNodeBaseComparer());
        
        idx = 1;
        foreach (SiteMapNodeBase node in nodes) {
            node.MenuOrder = idx;
            UpdateNodePosition(node.Id, node.ParentId, node.MenuOrder);
            idx += 2;
        }
    }

    private void MoveToDownLevel(bool isUp)
    {
        SiteMapProviderBase prov = GetTempSqlSiteMapProvider();
        SiteMapNodeBase selectedNode = (SiteMapNodeBase)
            prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);

        if ((isUp && selectedNode.PreviousSibling == null)
            || (!isUp && selectedNode.NextSibling == null))
            return;

        List<SiteMapNodeBase> nodes = new List<SiteMapNodeBase>();
        short idx = 1;
        SiteMapNodeBase destParentNode;
        if (isUp)
            destParentNode = (SiteMapNodeBase)selectedNode.PreviousSibling;
        else
            destParentNode = (SiteMapNodeBase)selectedNode.NextSibling;
        foreach (SiteMapNodeBase node in destParentNode.ChildNodes) {
            node.MenuOrder = idx;
            nodes.Add(node);
            idx += 2;
        }

        selectedNode.MenuOrder = short.MinValue;
        selectedNode.ParentId = destParentNode.Id;
        nodes.Add(selectedNode);
        nodes.Sort(new SiteMapNodeBaseComparer());
        
        idx = 1;
        foreach (SiteMapNodeBase node in nodes) {
            node.MenuOrder = idx;
            UpdateNodePosition(node.Id, node.ParentId, node.MenuOrder);
            idx += 2;
        }
    }

    private void MoveUpDown(bool isUp)
    {
        SiteMapProviderBase prov = GetTempSqlSiteMapProvider();
        SiteMapNodeBase selectedNode = (SiteMapNodeBase)
            prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);
        
        if (selectedNode.ParentNode == null)
            return;

        List<SiteMapNodeBase> nodes = new List<SiteMapNodeBase>();
        short idx = 1;
        foreach (SiteMapNodeBase node in selectedNode.ParentNode.ChildNodes) {
            node.MenuOrder = idx;
            nodes.Add(node);
            idx += 2;
        }
        if (isUp)
            selectedNode.MenuOrder -= 3;
        else
            selectedNode.MenuOrder += 3;

        nodes.Sort(new SiteMapNodeBaseComparer());
        
        idx = 1;
        foreach (SiteMapNodeBase node in nodes) {
            node.MenuOrder = idx;
            UpdateNodePosition(node.Id, node.ParentId, node.MenuOrder);
            idx += 2;
        }
    }

    private void UpdateNodePosition(int id, int parentId, short menuOrder)
    {
        string connStrName = ProviderHelper.GetConnectionStringName(SiteMap.Provider);
        string connStr = ConfigurationManager.ConnectionStrings[connStrName].ConnectionString;
        SqlConnection conn = new SqlConnection(connStr);
        SqlCommand cmd = new SqlCommand("SM_UpdateNodePosition", conn);

        cmd.Parameters.Add("@id", SqlDbType.Int).Value = id;
        cmd.Parameters.Add("@parentId", SqlDbType.Int).Value = parentId;
        cmd.Parameters.Add("@menuOrder", SqlDbType.SmallInt).Value = menuOrder;
        
        cmd.CommandType = CommandType.StoredProcedure;
        conn.Open();
        cmd.ExecuteNonQuery();
        conn.Close();
    }

    private void UpdateNodeData()
    {
        string connStrName = ProviderHelper.GetConnectionStringName(SiteMap.Provider);
        string connStr = ConfigurationManager.ConnectionStrings[connStrName].ConnectionString;
        SqlConnection conn = new SqlConnection(connStr);
        SqlCommand cmd = new SqlCommand("SM_UpdateNode", conn);

        cmd.Parameters.Add("@id", SqlDbType.Int).Value = int.Parse(hiddenID.Value);
        if (rblNodeType.SelectedIndex == 0) {
            cmd.Parameters.Add("@title", SqlDbType.VarChar, 100).Value = txtTitle.Text;
            cmd.Parameters.Add("@url", SqlDbType.VarChar, 255).Value = txtUrl.Text;
        }
        else {
            string childNodesDisplayMode = null;
            string boardNo = null;
            string nodeObjectParam;
            if (ddlChildNodesDisplayMode.SelectedIndex > 0)
                childNodesDisplayMode = ddlChildNodesDisplayMode.SelectedValue;
            boardNo = ddlBoard.SelectedValue;
            nodeObjectParam = "boardNo=" + boardNo;
            if (childNodesDisplayMode != null)
                nodeObjectParam += ";childNodesDisplayMode=" + childNodesDisplayMode;
            cmd.Parameters.Add("@nodeObjectParam", SqlDbType.VarChar, 100).Value = nodeObjectParam;
        }
        cmd.Parameters.Add("@iconFile", SqlDbType.VarChar, 255).Value = txtIconFile.Text;
        cmd.Parameters.Add("@description", SqlDbType.VarChar, 255).Value = txtDescription.Text;
        cmd.Parameters.Add("@roles", SqlDbType.VarChar, 100).Value = txtRoles.Text;
        cmd.Parameters.Add("@visible", SqlDbType.Bit).Value = chkVisible.Checked;
        cmd.Parameters.Add("@nodeObjectId", SqlDbType.Int).Value = int.Parse(rblNodeType.SelectedValue);
        
        cmd.CommandType = CommandType.StoredProcedure;
        conn.Open();
        cmd.ExecuteNonQuery();
        conn.Close();
    }

    private void AddNodeData(int parentId, short order)
    {
        string connStrName = ProviderHelper.GetConnectionStringName(SiteMap.Provider);
        string connStr = ConfigurationManager.ConnectionStrings[connStrName].ConnectionString;
        SqlConnection conn = new SqlConnection(connStr);
        SqlCommand cmd = new SqlCommand("SM_AddNode", conn);

        if (rblNodeType.SelectedIndex == 0) {
            cmd.Parameters.Add("@title", SqlDbType.VarChar, 100).Value = txtTitle.Text;
            cmd.Parameters.Add("@url", SqlDbType.VarChar, 255).Value = txtUrl.Text;
        }
        else {
            string childNodesDisplayMode = null;
            string boardNo = null;
            string nodeObjectParam;
            if (ddlChildNodesDisplayMode.SelectedIndex > 0)
                childNodesDisplayMode = ddlChildNodesDisplayMode.SelectedValue;
            boardNo = ddlBoard.SelectedValue;
            nodeObjectParam = "boardNo=" + boardNo;
            if (childNodesDisplayMode != null)
                nodeObjectParam += ";childNodesDisplayMode=" + childNodesDisplayMode;
            cmd.Parameters.Add("@nodeObjectParam", SqlDbType.VarChar, 100).Value = nodeObjectParam;
        }
        cmd.Parameters.Add("@iconFile", SqlDbType.VarChar, 255).Value = txtIconFile.Text;
        cmd.Parameters.Add("@description", SqlDbType.VarChar, 255).Value = txtDescription.Text;
        cmd.Parameters.Add("@parentId", SqlDbType.Int).Value = parentId;
        cmd.Parameters.Add("@menuOrder", SqlDbType.SmallInt).Value = order;
        cmd.Parameters.Add("@roles", SqlDbType.VarChar, 100).Value = txtRoles.Text;
        cmd.Parameters.Add("@visible", SqlDbType.Bit).Value = chkVisible.Checked;
        cmd.Parameters.Add("@nodeObjectId", SqlDbType.Int).Value = int.Parse(rblNodeType.SelectedValue);

        cmd.CommandType = CommandType.StoredProcedure;
        conn.Open();
        cmd.ExecuteNonQuery();
        conn.Close();
    }

    private void DeleteNode()
    {
        string connStrName = ProviderHelper.GetConnectionStringName(SiteMap.Provider);
        string connStr = ConfigurationManager.ConnectionStrings[connStrName].ConnectionString;
        SqlConnection conn = new SqlConnection(connStr);
        SqlCommand cmd = new SqlCommand("SM_DeleteNode", conn);

        cmd.Parameters.Add("@id", SqlDbType.Int).Value = int.Parse(hiddenID.Value);

        cmd.CommandType = CommandType.StoredProcedure;
        conn.Open();
        cmd.ExecuteNonQuery();
        conn.Close();
    }
    
    protected void btnUpdate_Click(object sender, EventArgs e)
    {
        SiteMapProviderBase prov = GetTempSqlSiteMapProvider();
        SiteMapNodeBase selectedNode = (SiteMapNodeBase)
            prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);

        SiteMapNode node;
        string searchUrl;
        if (rblNodeType.SelectedIndex == 0)
        {
            searchUrl = txtUrl.Text;
        }
        else {
            searchUrl = prov.BoardRelativeDirectory
                + Definations.ListPageName + "?"
                + Definations.BoardNoParmName + "=" 
                + ddlBoard.SelectedValue;
        }
        node = prov.FindSiteMapNode(searchUrl);
        if (node != null && !node.Equals(selectedNode)) {
            lblError.Text = "이미 존재하는 URL을 입력하셨습니다.";
            lblError.Visible = true;
            return;
        }

        UpdateNodeData();
        ProviderHelper.UpdateSiteMap(SiteMap.Provider);
        Response.Redirect(Request.RawUrl);
    }
    
    protected void btnAdd_Click(object sender, EventArgs e)
    {
        SiteMapProviderBase prov = GetTempSqlSiteMapProvider();
        SiteMapNodeBase selectedNode = (SiteMapNodeBase)
            prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);

        SiteMapNode node;
        string searchUrl;
        if (rblNodeType.SelectedIndex == 0) {
            searchUrl = txtUrl.Text;
        }
        else {
            searchUrl = prov.BoardRelativeDirectory
                + Definations.ListPageName + "?"
                + Definations.BoardNoParmName + "="
                + ddlBoard.SelectedValue;
        }
        node = prov.FindSiteMapNode(searchUrl);
        if (node != null) {
            lblError.Text = "이미 존재하는 URL을 입력하셨습니다.";
            lblError.Visible = true;
            return;
        }

        int order = 1;
        if (selectedNode.HasChildNodes)
            order = ((SiteMapNodeBase)selectedNode.ChildNodes[selectedNode.ChildNodes.Count - 1]).MenuOrder + 1;

        AddNodeData(selectedNode.Id, (short)order);
        ProviderHelper.UpdateSiteMap(SiteMap.Provider);
        Response.Redirect(Request.RawUrl);
    }

    protected void btnDelete_Click(object sender, EventArgs e)
    {
        SiteMapProviderBase prov = GetTempSqlSiteMapProvider();
        SiteMapNodeBase selectedNode = (SiteMapNodeBase)
            prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);

        if (selectedNode.HasChildNodes) {
            lblError.Text = "자식노드가 있어서 삭제 할 수 없습니다.";
            lblError.Visible = true;
            return;
        }

        DeleteNode();
        ProviderHelper.UpdateSiteMap(SiteMap.Provider);
        Response.Redirect(Request.RawUrl);        
    }

    protected void btnClose_Click(object sender, EventArgs e)
    {
        Response.Redirect(Request.RawUrl);
    }

    protected void rblNodeType_SelectedIndexChanged(object sender, EventArgs e)
    {
        if (rblNodeType.SelectedValue.Equals("1")) {
            txtTitle.Enabled = true;
            txtUrl.Enabled = true;
            txtDescription.Enabled = true;
            RequiredFieldValidator1.Enabled = true;
            ddlChildNodesDisplayMode.Enabled = false;
            ddlBoard.Enabled = false;
        }
        else {
            txtTitle.Enabled = false;
            txtUrl.Enabled = false;
            txtDescription.Enabled = false;
            RequiredFieldValidator1.Enabled = false;
            ddlChildNodesDisplayMode.Enabled = true;
            ddlBoard.Enabled = true;
        }
    }
</script>
<asp:Panel ID="pnlEditNode" runat="server" GroupingText="선택된 노드 편집" Visible="False" Width="350px" DefaultButton="btnUpdate" >
    이동 :
    <asp:ImageButton ID="ibtnLeftUp" runat="server" ToolTip="상위노드의 위로 이동" OnCommand="ibtnMove_Command" CommandName="leftUp" CausesValidation="false" />&nbsp;
    <asp:ImageButton ID="ibtnLeftDown" runat="server" ToolTip="상위노드의 아래로 이동" OnCommand="ibtnMove_Command" CommandName="leftDown" CausesValidation="false" />&nbsp;
    <asp:ImageButton ID="ibtnUp" runat="server" ToolTip="위로 이동" OnCommand="ibtnMove_Command" CommandName="up" CausesValidation="false" />&nbsp;
    <asp:ImageButton ID="ibtnDown" runat="server" ToolTip="아래로 이동" OnCommand="ibtnMove_Command" CommandName="down" CausesValidation="false" />&nbsp;
    <asp:ImageButton ID="ibtnUpRight" runat="server" ToolTip="위 노드의 하위로 이동" OnCommand="ibtnMove_Command" CommandName="upRight" CausesValidation="false" />&nbsp;
    <asp:ImageButton ID="ibtnDownRight" runat="server" ToolTip="아래 노드의 하위로 이동" OnCommand="ibtnMove_Command" CommandName="downRight" CausesValidation="false" /><br />
    <asp:RadioButtonList ID="rblNodeType" runat="server" AutoPostBack="True" 
        OnSelectedIndexChanged="rblNodeType_SelectedIndexChanged"
        RepeatDirection="Horizontal" DataSourceID="sdsNodeType" DataTextField="name" DataValueField="id" >
    </asp:RadioButtonList>
    <asp:SqlDataSource ID="sdsNodeType" runat="server" 
        ConnectionString="<%$ ConnectionStrings:MySqlConnection %>"
        SelectCommand="SM_GetNodeDefinations" 
        SelectCommandType="StoredProcedure" />
    <br />
    제목 :<asp:TextBox ID="txtTitle" runat="server" />
    <asp:RequiredFieldValidator ID="RequiredFieldValidator1" runat="server" ErrorMessage="<br>제목을 입력하세요." ControlToValidate="txtTitle" Display="Dynamic" />
    <br />
    URL :<asp:TextBox ID="txtUrl" runat="server" Columns="40" />
    <br />
    설명 :<asp:TextBox ID="txtDescription" runat="server" Columns="40" /><br />
    출력 :<asp:CheckBox ID="chkVisible" runat="server" Text="나타남" /><br />
    아이콘 :<asp:TextBox ID="txtIconFile" runat="server" Columns="40" /><br />
    역할 :<asp:TextBox ID="txtRoles" runat="server" Columns="40" /> (예:admin,users)<br />
    자식노드출력형식 :
    <asp:DropDownList ID="ddlChildNodesDisplayMode" runat="server" >
        <asp:ListItem>설정안함</asp:ListItem>
        <asp:ListItem>None</asp:ListItem>
        <asp:ListItem>SupportSiteMapPath</asp:ListItem>
        <asp:ListItem>EnumerateContentTitle</asp:ListItem>
    </asp:DropDownList><br />
    게시판 선택 :<asp:DropDownList ID="ddlBoard" runat="server" 
        DataSourceID="sdsBoard" DataTextField="subject" DataValueField="no" />
    <asp:SqlDataSource ID="sdsBoard" runat="server" 
        ConnectionString="<%$ ConnectionStrings:MySqlConnection %>"
        SelectCommand="SM_GetBoardInfo" SelectCommandType="StoredProcedure" />
    <asp:HiddenField ID="hiddenID" runat="server" />
    <br />
    <asp:Button ID="btnUpdate" runat="server" Text="갱신" OnClick="btnUpdate_Click" />
    <asp:Button ID="btnAdd" runat="server" Text="추가" OnClick="btnAdd_Click" />
    <asp:Button ID="btnDelete" runat="server" Text="삭제" OnClick="btnDelete_Click" />
    <asp:Button ID="btnClose" runat="server" Text="닫기" OnClick="btnClose_Click" />
    <br />
    <asp:Label ID="lblError" runat="server" ForeColor="Red" Visible="false" />
</asp:Panel>
<asp:TreeView ID="TreeView1" runat="server" ShowLines="True" OnSelectedNodeChanged="TreeView1_SelectedNodeChanged" >
    <SelectedNodeStyle BorderColor="Blue" BorderWidth="2px" />
</asp:TreeView>

