using System;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Security.Permissions;
using System.Web;

namespace SqlSiteMapProvider
{
    public class SsmSiteMapNode : SiteMapNode { 
        private int _id = -1;
        private int _parentId;
        private short _menuOrder;
        private string _iconFile;
        private bool _visible = true;

        public int Id {
            get { return _id; }
        }
        public int ParentId {
            get { return _parentId; }
            set { _parentId = value; }
        }
        public short MenuOrder {
            get { return _menuOrder; }
            set { _menuOrder = value; }
        }
        public string IconFile {
            get { return _iconFile; }
            set { _iconFile = value; }
        }
        public bool Visible {
            get { return _visible; }
            set { _visible = value; }
        }

        public SsmSiteMapNode(SiteMapProvider provider, string key)
            : base(provider, key) { }

        public SsmSiteMapNode(SiteMapProvider provider, int id, string key) 
            : base(provider, key) {
                _id = id;
        }
    }

    public class SsmSiteMapNodeComparer : IComparer<SsmSiteMapNode>
    {
        public int Compare(SsmSiteMapNode x, SsmSiteMapNode y) {
            return x.MenuOrder.CompareTo(y.MenuOrder);
        }
    }

    public class SqlSiteMapProvider : StaticSiteMapProvider
    {
        private SsmSiteMapNode _rootNode = null;
        private string _connStr = null;
        private string _connStrName = null;
        private bool _initialized = false;
        private bool _isNotEditMode = true;

        public SqlSiteMapProvider() { }

        public SqlSiteMapProvider(bool notEditable) {
            _isNotEditMode = notEditable;
        }

        internal string ConnectionStringName {
            get { return _connStrName; }
        }

        protected override SiteMapNode GetRootNodeCore()
        {
            return BuildSiteMap();
        }

        public override void Initialize(string name, NameValueCollection attributes)
        {
            if (_initialized)
                throw new InvalidOperationException("ʱȭ ι ̻   ϴ.");

            _connStrName = attributes["connectionStringName"];
            if (string.IsNullOrEmpty(_connStrName))
                throw new ConfigurationErrorsException("ι̴  connectionStringName Ӽ  ʾҽϴ");

            _connStr = ConfigurationManager.ConnectionStrings[_connStrName].ConnectionString;

            base.Initialize(name, attributes);
            _initialized = true;
        }

        protected override void Clear()
        {
            lock (this) {
                _rootNode = null;
                base.Clear();
            }
        }

        public override SiteMapNode BuildSiteMap()
        {
            lock (this) {
                if (!_initialized)
                    throw new Exception("ʱȭ  BuildSiteMap ޼带 ȣ Ҽ ϴ.");

                if (_rootNode != null)
                    return _rootNode;

                Clear();

                int rootNodeId = -1;
                SqlConnection conn = new SqlConnection(_connStr);
                SqlCommand cmd = new SqlCommand("SSM_GetRootNode", conn);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("@checkVisible", SqlDbType.Bit).Value = _isNotEditMode;

                conn.Open();
                SqlDataReader dr = cmd.ExecuteReader();
                if (dr.Read())  {
                    rootNodeId = (int)dr["id"];
                    string url = (string)dr["url"];

                    _rootNode = new SsmSiteMapNode(this, rootNodeId, 
                        string.IsNullOrEmpty(url)? rootNodeId.ToString(): url);
                    _rootNode.Url = url;
                    if (_rootNode.Url.Equals(string.Empty))
                        _rootNode.Url = null;
                    _rootNode.Title = (string)dr["title"];
                    _rootNode.Description = dr["description"] == DBNull.Value ? "" : (string)dr["description"];
                    _rootNode.IconFile = dr["iconFile"] == DBNull.Value ? "" : (string)dr["iconFile"];
                    _rootNode.MenuOrder = (short)dr["menuOrder"];
                    _rootNode.ParentId = (int)dr["parentId"];
                    _rootNode.Visible = (bool)dr["visible"];
                    _rootNode.ReadOnly = _isNotEditMode;
                    AddNode(_rootNode);
                }
                dr.Close();
                conn.Close();

                if (_rootNode != null)
                    AddChildren(_rootNode, rootNodeId);

                return _rootNode;
            }
        }

        private void AddChildren(SiteMapNode parentNode, int parentID)
        {
            SqlConnection conn = new SqlConnection(_connStr);
            SqlCommand cmd = new SqlCommand("SSM_GetNodeByParentId");
            SqlDataAdapter adapter = new SqlDataAdapter();

            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Connection = conn;
            cmd.Parameters.Add("@parentId", SqlDbType.Int).Value = parentID;
            cmd.Parameters.Add("@checkVisible", SqlDbType.Bit).Value = _isNotEditMode;
            
            adapter.SelectCommand = cmd;

            DataTable dt = new DataTable();
            adapter.Fill(dt);
            foreach (DataRow row in dt.Rows) {
                int id = (int)row["id"];
                string url = (string)row["url"];

                SsmSiteMapNode node = new SsmSiteMapNode(this, id,
                    string.IsNullOrEmpty(url) ? id.ToString() : url);
                node.Url = url;
                if (node.Url.Equals(string.Empty))
                    node.Url = null;
                node.Title = (string)row["title"];
                node.Description = row["description"] == DBNull.Value ? "" : (string)row["description"];
                node.IconFile = row["iconFile"] == DBNull.Value ? "" : (string)row["iconFile"];
                node.MenuOrder = (short)row["menuOrder"];
                node.ParentId = (int)row["parentId"];
                node.Visible = (bool)row["visible"];
                node.ReadOnly = _isNotEditMode;

                AddNode(node, parentNode);
                AddChildren(node, id);
            }
        }

        internal void UpdateSiteMap()
        {
            _rootNode = null;
            BuildSiteMap();
        }
    }
}
