﻿<%@ Control Language="C#" ClassName="EditSiteMap" %>
<%@ Import Namespace="System.Configuration" %>
<%@ Import Namespace="SqlSiteMapProvider" %>
<%@ Import Namespace="System.Data" %>
<%@ Import Namespace="System.Data.SqlClient" %>
<%@ Import Namespace="System.Collections.Generic" %>
<script runat="server">
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack) {
            CreateSiteMapTreeView();
        }
    }

    private string _selectedValue = null;
    private void CreateSiteMapTreeView() {
        SqlSiteMapProvider prov = GetTempSqlSiteMapProvider();
        SsmSiteMapNode rootNode = (SsmSiteMapNode)prov.RootNode;

        _selectedValue = TreeView1.SelectedValue;
        TreeView1.Nodes.Clear();
        TreeNode treeRootNode = CreateTreeNode(rootNode);
        TreeView1.Nodes.Add(treeRootNode);
        CreateTreeNodes(rootNode, treeRootNode);
        TreeView1.ExpandAll();
    }

    private SqlSiteMapProvider GetTempSqlSiteMapProvider() {
        NameValueCollection atts = new NameValueCollection();
        atts.Add("connectionStringName", ProviderHelper.GetConnectionStringName(SiteMap.Provider));
        SqlSiteMapProvider prov = new SqlSiteMapProvider(false);
        prov.Initialize(SiteMap.Provider.Name, atts);

        return prov;
    }

    private TreeNode CreateTreeNode(SsmSiteMapNode node)
    {
        TreeNode treeNode = new TreeNode(node.Title, node.Key);
        treeNode.ImageUrl = node.IconFile;
        treeNode.ToolTip = node.Description;
        if (!node.Visible)
            treeNode.Text += "<font color='red'><i>(숨김)</i></font>";

        treeNode.Selected = !string.IsNullOrEmpty(_selectedValue)
            && _selectedValue.Equals(node.Key);
        return treeNode;
    }

    private void CreateTreeNodes(SsmSiteMapNode parentSsmn, TreeNode parentNode)
    {
        if (parentSsmn.HasChildNodes) {
            foreach (SsmSiteMapNode n in parentSsmn.ChildNodes) {
                TreeNode node = CreateTreeNode(n);

                parentNode.ChildNodes.Add(node);
                if (n.HasChildNodes)
                    CreateTreeNodes(n, node);
            }
        }
    }

    protected void TreeView1_SelectedNodeChanged(object sender, EventArgs e)
    {
        SqlSiteMapProvider prov = GetTempSqlSiteMapProvider();
        SsmSiteMapNode selectedNode = 
            (SsmSiteMapNode)prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);

        bool isRoot = false;
        if (prov.RootNode.Key.Equals(TreeView1.SelectedValue)) {
            isRoot = true;
        }
        else if (selectedNode == null)  {
            pnlEditNode.Visible = false;
            return;
        }

        if (!isRoot && selectedNode.PreviousSibling != null) {
            ibtnUp.Enabled = true;
            ibtnUp.ImageUrl = "~/Images/blue13_up.gif";
            ibtnUpRight.Enabled = true;
            ibtnUpRight.ImageUrl = "~/Images/blue13_upRight.gif";
        }
        else {
            ibtnUp.Enabled = false;
            ibtnUp.ImageUrl = "~/Images/gray05_up.gif";
            ibtnUpRight.Enabled = false;
            ibtnUpRight.ImageUrl = "~/Images/gray05_upRight.gif";
        }
        if (!isRoot && selectedNode.NextSibling != null) {
            ibtnDown.Enabled = true;
            ibtnDown.ImageUrl = "~/Images/blue13_down.gif";
            ibtnDownRight.Enabled = true;
            ibtnDownRight.ImageUrl = "~/Images/blue13_downRight.gif";
        }
        else {
            ibtnDown.Enabled = false;
            ibtnDown.ImageUrl = "~/Images/gray05_down.gif";
            ibtnDownRight.Enabled = false;
            ibtnDownRight.ImageUrl = "~/Images/gray05_downRight.gif";
        }
        if (!isRoot && selectedNode.ParentNode != null 
            && selectedNode.ParentNode.ParentNode != null) {
            ibtnLeftUp.Enabled = true;
            ibtnLeftUp.ImageUrl = "~/Images/blue13_leftUp.gif";
            ibtnLeftDown.Enabled = true;
            ibtnLeftDown.ImageUrl = "~/Images/blue13_leftDown.gif";
        }
        else {
            ibtnLeftUp.Enabled = false;
            ibtnLeftUp.ImageUrl = "~/Images/gray05_leftUp.gif";
            ibtnLeftDown.Enabled = false;
            ibtnLeftDown.ImageUrl = "~/Images/gray05_leftDown.gif";
        }

        txtTitle.Text = selectedNode.Title;
        txtUrl.Text = selectedNode.Url;
        txtDescription.Text = selectedNode.Description;
        chkVisible.Checked = selectedNode.Visible;
        txtIconFile.Text = selectedNode.IconFile;
        hiddenID.Value = selectedNode.Id.ToString();
        pnlEditNode.Visible = true;
    }

    protected void ibtnMove_Command(object sender, CommandEventArgs e)
    {
        switch (e.CommandName) {
            case "leftUp":
                MoveToUpLevel(true);
                break;
            case "leftDown":
                MoveToUpLevel(false);
                break;
            case "up":
                MoveUpDown(true);
                break;
            case "down":
                MoveUpDown(false);
                break;
            case "upRight":
                MoveToDownLevel(true);
                break;
            case "downRight":
                MoveToDownLevel(false);
                break;
        }
        CreateSiteMapTreeView();
        ProviderHelper.UpdateSiteMap(SiteMap.Provider);
        TreeView1_SelectedNodeChanged(null, null);
    }

    private void MoveToUpLevel(bool isUp)
    {
        SqlSiteMapProvider prov = GetTempSqlSiteMapProvider();
        SsmSiteMapNode selectedNode = (SsmSiteMapNode)
            prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);

        if (selectedNode.ParentNode == null 
            || selectedNode.ParentNode.ParentNode == null)
            return;

        List<SsmSiteMapNode> nodes = new List<SsmSiteMapNode>();
        short idx = 1;
        SsmSiteMapNode parentNode = (SsmSiteMapNode)selectedNode.ParentNode;
        foreach (SsmSiteMapNode node in parentNode.ParentNode.ChildNodes) {
            node.MenuOrder = idx;
            nodes.Add(node);
            idx += 2;
        }
        if (isUp)
            selectedNode.MenuOrder = (short)(parentNode.MenuOrder - 1);
        else
            selectedNode.MenuOrder = (short)(parentNode.MenuOrder + 1);

        selectedNode.ParentId = parentNode.ParentId;
        nodes.Add(selectedNode);
        nodes.Sort(new SsmSiteMapNodeComparer());
        
        idx = 1;
        foreach (SsmSiteMapNode node in nodes) {
            node.MenuOrder = idx;
            UpdateNodePosition(node.Id, node.ParentId, node.MenuOrder);
            idx += 2;
        }
    }

    private void MoveToDownLevel(bool isUp)
    {
        SqlSiteMapProvider prov = GetTempSqlSiteMapProvider();
        SsmSiteMapNode selectedNode = (SsmSiteMapNode)
            prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);

        if ((isUp && selectedNode.PreviousSibling == null)
            || (!isUp && selectedNode.NextSibling == null))
            return;

        List<SsmSiteMapNode> nodes = new List<SsmSiteMapNode>();
        short idx = 1;
        SsmSiteMapNode destParentNode;
        if (isUp)
            destParentNode = (SsmSiteMapNode)selectedNode.PreviousSibling;
        else
            destParentNode = (SsmSiteMapNode)selectedNode.NextSibling;
        foreach (SsmSiteMapNode node in destParentNode.ChildNodes) {
            node.MenuOrder = idx;
            nodes.Add(node);
            idx += 2;
        }

        selectedNode.MenuOrder = short.MinValue;
        selectedNode.ParentId = destParentNode.Id;
        nodes.Add(selectedNode);
        nodes.Sort(new SsmSiteMapNodeComparer());
        
        idx = 1;
        foreach (SsmSiteMapNode node in nodes) {
            node.MenuOrder = idx;
            UpdateNodePosition(node.Id, node.ParentId, node.MenuOrder);
            idx += 2;
        }
    }

    private void MoveUpDown(bool isUp)
    {
        SqlSiteMapProvider prov = GetTempSqlSiteMapProvider();
        SsmSiteMapNode selectedNode = (SsmSiteMapNode)
            prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);
        
        if (selectedNode.ParentNode == null)
            return;

        List<SsmSiteMapNode> nodes = new List<SsmSiteMapNode>();
        short idx = 1;
        foreach (SsmSiteMapNode node in selectedNode.ParentNode.ChildNodes) {
            node.MenuOrder = idx;
            nodes.Add(node);
            idx += 2;
        }
        if (isUp)
            selectedNode.MenuOrder -= 3;
        else
            selectedNode.MenuOrder += 3;

        nodes.Sort(new SsmSiteMapNodeComparer());
        
        idx = 1;
        foreach (SsmSiteMapNode node in nodes) {
            node.MenuOrder = idx;
            UpdateNodePosition(node.Id, node.ParentId, node.MenuOrder);
            idx += 2;
        }
    }

    private void UpdateNodePosition(int id, int parentId, short menuOrder)
    {
        string connStrName = ProviderHelper.GetConnectionStringName(SiteMap.Provider);
        string connStr = ConfigurationManager.ConnectionStrings[connStrName].ConnectionString;
        SqlConnection conn = new SqlConnection(connStr);
        SqlCommand cmd = new SqlCommand("SSM_UpdateNodePosition", conn);

        cmd.Parameters.Add("@id", SqlDbType.Int).Value = id;
        cmd.Parameters.Add("@parentId", SqlDbType.Int).Value = parentId;
        cmd.Parameters.Add("@menuOrder", SqlDbType.SmallInt).Value = menuOrder;
        
        cmd.CommandType = CommandType.StoredProcedure;
        conn.Open();
        cmd.ExecuteNonQuery();
        conn.Close();
    }

    private void UpdateNodeData()
    {
        string connStrName = ProviderHelper.GetConnectionStringName(SiteMap.Provider);
        string connStr = ConfigurationManager.ConnectionStrings[connStrName].ConnectionString;
        SqlConnection conn = new SqlConnection(connStr);
        SqlCommand cmd = new SqlCommand("SSM_UpdateNode", conn);

        cmd.Parameters.Add("@id", SqlDbType.Int).Value = int.Parse(hiddenID.Value);
        cmd.Parameters.Add("@title", SqlDbType.VarChar, 100).Value = txtTitle.Text;
        cmd.Parameters.Add("@url", SqlDbType.VarChar, 255).Value = txtUrl.Text;
        cmd.Parameters.Add("@iconFile", SqlDbType.VarChar, 255).Value = txtIconFile.Text;
        cmd.Parameters.Add("@description", SqlDbType.VarChar, 255).Value = txtDescription.Text;
        cmd.Parameters.Add("@visible", SqlDbType.Bit).Value = chkVisible.Checked;
        
        cmd.CommandType = CommandType.StoredProcedure;
        conn.Open();
        cmd.ExecuteNonQuery();
        conn.Close();
    }

    private void AddNodeData(int parentId, short order)
    {
        string connStrName = ProviderHelper.GetConnectionStringName(SiteMap.Provider);
        string connStr = ConfigurationManager.ConnectionStrings[connStrName].ConnectionString;
        SqlConnection conn = new SqlConnection(connStr);
        SqlCommand cmd = new SqlCommand("SSM_AddNode", conn);

        cmd.Parameters.Add("@title", SqlDbType.VarChar, 100).Value = txtTitle.Text;
        cmd.Parameters.Add("@url", SqlDbType.VarChar, 255).Value = txtUrl.Text;
        cmd.Parameters.Add("@iconFile", SqlDbType.VarChar, 255).Value = txtIconFile.Text;
        cmd.Parameters.Add("@description", SqlDbType.VarChar, 255).Value = txtDescription.Text;
        cmd.Parameters.Add("@parentId", SqlDbType.Int).Value = parentId;
        cmd.Parameters.Add("@menuOrder", SqlDbType.SmallInt).Value = order;
        cmd.Parameters.Add("@visible", SqlDbType.Bit).Value = chkVisible.Checked;

        cmd.CommandType = CommandType.StoredProcedure;
        conn.Open();
        cmd.ExecuteNonQuery();
        conn.Close();
    }

    private void DeleteNode()
    {
        string connStrName = ProviderHelper.GetConnectionStringName(SiteMap.Provider);
        string connStr = ConfigurationManager.ConnectionStrings[connStrName].ConnectionString;
        SqlConnection conn = new SqlConnection(connStr);
        SqlCommand cmd = new SqlCommand("SSM_DeleteNode", conn);

        cmd.Parameters.Add("@id", SqlDbType.Int).Value = int.Parse(hiddenID.Value);

        cmd.CommandType = CommandType.StoredProcedure;
        conn.Open();
        cmd.ExecuteNonQuery();
        conn.Close();
    }
    
    protected void btnUpdate_Click(object sender, EventArgs e)
    {
        SqlSiteMapProvider prov = GetTempSqlSiteMapProvider();
        SsmSiteMapNode selectedNode = (SsmSiteMapNode)
            prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);

        SiteMapNode node = prov.FindSiteMapNode(txtUrl.Text);
        if (node != null && !node.Equals(selectedNode)) {
            lblError.Text = "이미 존재하는 URL을 입력하셨습니다.";
            lblError.Visible = true;
            return;
        }

        UpdateNodeData();
        ProviderHelper.UpdateSiteMap(SiteMap.Provider);
        Response.Redirect(Request.RawUrl);
    }
    
    protected void btnAdd_Click(object sender, EventArgs e)
    {
        SqlSiteMapProvider prov = GetTempSqlSiteMapProvider();
        SsmSiteMapNode selectedNode = (SsmSiteMapNode)
            prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);

        SiteMapNode node = prov.FindSiteMapNode(txtUrl.Text);
        if (node != null) {
            lblError.Text = "이미 존재하는 URL을 입력하셨습니다.";
            lblError.Visible = true;
            return;
        }

        int order = 1;
        if (selectedNode.HasChildNodes)
            order = ((SsmSiteMapNode)selectedNode.ChildNodes[selectedNode.ChildNodes.Count - 1]).MenuOrder + 1;

        AddNodeData(selectedNode.Id, (short)order);
        ProviderHelper.UpdateSiteMap(SiteMap.Provider);
        Response.Redirect(Request.RawUrl);
    }

    protected void btnDelete_Click(object sender, EventArgs e)
    {
        SqlSiteMapProvider prov = GetTempSqlSiteMapProvider();
        SsmSiteMapNode selectedNode = (SsmSiteMapNode)
            prov.FindSiteMapNodeFromKey(TreeView1.SelectedValue);

        if (selectedNode.HasChildNodes) {
            lblError.Text = "자식노드가 있어서 삭제 할 수 없습니다.";
            lblError.Visible = true;
            return;
        }

        DeleteNode();
        ProviderHelper.UpdateSiteMap(SiteMap.Provider);
        Response.Redirect(Request.RawUrl);        
    }

    protected void btnClose_Click(object sender, EventArgs e)
    {
        Response.Redirect(Request.RawUrl);
    }
</script>
<asp:Panel ID="pnlEditNode" runat="server" GroupingText="선택된 노드 편집" Visible="False" Width="350px" DefaultButton="btnUpdate" >
    이동 :
    <asp:ImageButton ID="ibtnLeftUp" runat="server" ToolTip="상위노드의 위로 이동" OnCommand="ibtnMove_Command" CommandName="leftUp" CausesValidation="false" />&nbsp;
    <asp:ImageButton ID="ibtnLeftDown" runat="server" ToolTip="상위노드의 아래로 이동" OnCommand="ibtnMove_Command" CommandName="leftDown" CausesValidation="false" />&nbsp;
    <asp:ImageButton ID="ibtnUp" runat="server" ToolTip="위로 이동" OnCommand="ibtnMove_Command" CommandName="up" CausesValidation="false" />&nbsp;
    <asp:ImageButton ID="ibtnDown" runat="server" ToolTip="아래로 이동" OnCommand="ibtnMove_Command" CommandName="down" CausesValidation="false" />&nbsp;
    <asp:ImageButton ID="ibtnUpRight" runat="server" ToolTip="위 노드의 하위로 이동" OnCommand="ibtnMove_Command" CommandName="upRight" CausesValidation="false" />&nbsp;
    <asp:ImageButton ID="ibtnDownRight" runat="server" ToolTip="아래 노드의 하위로 이동" OnCommand="ibtnMove_Command" CommandName="downRight" CausesValidation="false" /><br />
    제목 :<asp:TextBox ID="txtTitle" runat="server" />
    <asp:RequiredFieldValidator ID="RequiredFieldValidator1" runat="server" ErrorMessage="<br>제목을 입력하세요." ControlToValidate="txtTitle" Display="Dynamic" />
    <br />
    URL :<asp:TextBox ID="txtUrl" runat="server" Columns="40" /><br />
    설명 :<asp:TextBox ID="txtDescription" runat="server" Columns="40" /><br />
    출력 :<asp:CheckBox ID="chkVisible" runat="server" Text="나타남" /><br />
    아이콘 :<asp:TextBox ID="txtIconFile" runat="server" Columns="40" />
    <asp:HiddenField ID="hiddenID" runat="server" />
    <br />
    <asp:Button ID="btnUpdate" runat="server" Text="갱신" OnClick="btnUpdate_Click" />
    <asp:Button ID="btnAdd" runat="server" Text="추가" OnClick="btnAdd_Click" />
    <asp:Button ID="btnDelete" runat="server" Text="삭제" OnClick="btnDelete_Click" />
    <asp:Button ID="btnClose" runat="server" Text="닫기" OnClick="btnClose_Click" />
    <br />
    <asp:Label ID="lblError" runat="server" ForeColor="Red" Visible="false" /></asp:Panel>
<asp:TreeView ID="TreeView1" runat="server" ShowLines="True" OnSelectedNodeChanged="TreeView1_SelectedNodeChanged" >
    <SelectedNodeStyle BorderColor="Blue" BorderWidth="2px" />
</asp:TreeView>

