using System;
using System.Collections;
using System.Web.Configuration;
using System.Configuration;
using System.Configuration.Provider;

namespace SimpleLog
{
    public class SimpleLog {
        private static SimpleLogProviderCollection s_Providers = null;
        private static SimpleLogProvider s_Provider = null;
        private static bool s_Initialized = false;
        private static object s_lock = new object();
        private static Exception s_InitializeException = null;

        public static SimpleLogProviderCollection Providers {
            get {
                Initialize();
                return s_Providers;
            }
        }

        public static string ProviderName {
            get {
                Initialize();
                return s_Provider.Name;
            }
        }

        public static string ProviderDescription {
            get {
                Initialize();
                return s_Provider.Description;
            }
        }

        private static void Initialize() 
        {
            if (s_Initialized) {
                if (s_InitializeException != null)
                    throw s_InitializeException;
                else
                    return;
            }
            lock (s_lock) {
                try {
                    SimpleLogConfigurationHandler section =
                        (SimpleLogConfigurationHandler)
                        ConfigurationManager.GetSection("system.web/SimpleLog");

                    if (section.Providers == null 
                        || section.Providers.Count < 1)
                        throw new ProviderException("SimpleLog ι̴  ùٸ ʽϴ.");

                    s_Providers = new SimpleLogProviderCollection();
                    ProvidersHelper.InstantiateProviders(
                        section.Providers, s_Providers, 
                        typeof(SimpleLogProvider));
                    s_Providers.SetReadOnly();
                    if (section.DefaultProvider == null)
                        s_InitializeException = ((Exception)
                            new ProviderException("⺻ι̴  ʾҽϴ."));
                    else {
                        try {
                            s_Provider = (SimpleLogProvider)s_Providers[section.DefaultProvider];
                        }
                        catch { }
                        if (s_Provider == null)
                            s_InitializeException = (Exception)
                                new ConfigurationErrorsException(
                                " ⺻ι̴  ʽϴ", 
                                section.ElementInformation.Properties["defaultProvider"].Source, 
                                section.ElementInformation.Properties["defaultProvider"].LineNumber);
                    }
                    s_Initialized = true;
                }
                catch (Exception err) {
                    s_InitializeException = err;
                }
                if (s_InitializeException != null)
                    throw s_InitializeException;
            }
        }

        public static ICollection GetLogList() {
            Initialize();
            return s_Provider.GetLogList();
        }

        public static void WriteLog(string pageName, string userHostAddress) {
            Initialize();
            s_Provider.WriteLog(pageName, userHostAddress);
        }
    }
}
