using System;
using System.Collections;
using System.Collections.Specialized;
using System.Configuration;
using System.Xml;

namespace SimpleLogB
{
    class SimpleLogConfiguration
    {
        string _defaultProvider;
        Hashtable _providers;

        public SimpleLogConfiguration() {
            _providers = new Hashtable();
        }

        public static SimpleLogConfiguration GetConfig()
        {
            // ⿡ IConfigurationSectionHandler.Create ޼ҵ尡 ȣȴ.
            return (SimpleLogConfiguration)ConfigurationManager.GetSection("system.web/SimpleLog");
        }

        public void LoadValuesFromConfigurationXml(XmlNode node)
        {
            XmlAttributeCollection attributeCollection = node.Attributes;

            // Get the default provider
            _defaultProvider = attributeCollection["defaultProvider"].Value;

            // Read child nodes
            foreach (XmlNode child in node.ChildNodes)
            {
                if (child.Name == "providers")
                    GetProviders(child);
            }
        }

        void GetProviders(XmlNode node)
        {
            foreach (XmlNode provider in node.ChildNodes)
            {
                switch (provider.Name)
                {
                    case "add":
                        _providers.Add(provider.Attributes["name"].Value, new ProviderData(provider.Attributes));
                        break;

                    case "remove":
                        _providers.Remove(provider.Attributes["name"].Value);
                        break;

                    case "clear":
                        _providers.Clear();
                        break;
                }
            }
        }

        public string DefaultProvider { get { return _defaultProvider; } }
        public Hashtable Providers { get { return _providers; } } 
    }

    public class ProviderData
    {
        string name;
        string providerType;
        NameValueCollection providerAttributes;

        public ProviderData(XmlAttributeCollection attributes)
        {
            name = attributes["name"].Value;
            providerType = attributes["type"].Value;

            providerAttributes = new NameValueCollection();
            foreach (XmlAttribute attribute in attributes)
            {
                if ((attribute.Name != "name") && (attribute.Name != "type"))
                    providerAttributes.Add(attribute.Name, attribute.Value);
            }

        }

        public string Name {
            get {
                return name;
            }
        }

        public string Type {
            get {
                return providerType;
            }
        }

        public NameValueCollection Attributes {
            get {
                return providerAttributes;
            }
        }
    }
}
