using System;
using System.Web;
using System.Reflection;
using System.Web.Caching;
using System.Collections;
using System.Collections.Specialized;
using System.Configuration.Provider;

namespace SimpleLogB
{
    public abstract class SimpleLogProvider : ProviderBase
    {
        public static SimpleLogProvider Instance()
        {
            // Use the cache because the reflection used later is expensive
            Cache cache = HttpContext.Current.Cache;
            Type type = null;
            string cacheKey = null;

            // Get the names of the providers
            SimpleLogConfiguration config = SimpleLogConfiguration.GetConfig();

            // Read the configuration specific information
            // for this provider
            ProviderData simpleLogProvider = (ProviderData)config.Providers[config.DefaultProvider];

            // In the cache?
            cacheKey = "SimpleLog::" + config.DefaultProvider;
            if (cache[cacheKey] == null)
            {
                // The assembly should be in \bin or GAC, so we simply need
                // to get an instance of the type
                try {
                    type = Type.GetType(simpleLogProvider.Type);

                    // Insert the type into the cache
                    Type[] paramTypes = new Type[2];
                    paramTypes[0] = typeof(string);
                    paramTypes[1] = typeof(NameValueCollection);

                    cache.Insert(cacheKey, type.GetConstructor(paramTypes));
                }
                catch (Exception e)  {
                    throw new Exception("Unable to load provider", e);
                }
            }

            // Load the configuration settings
            object[] paramArray = new object[2];
            paramArray[0] = simpleLogProvider.Name;
            paramArray[1] = simpleLogProvider.Attributes;


            ConstructorInfo constInfo = (ConstructorInfo)cache[cacheKey];

            return (SimpleLogProvider)(constInfo.Invoke(paramArray));

            //return (SimpleLogProvider)Activator.CreateInstance(type, paramArray);
        }

        public abstract ICollection GetLogList();
        public abstract void WriteLog(string pageName, string userHostAddress);
    }
}
